# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional

from scaleway_core.bridge import (
    Zone as ScwZone,
)
from scaleway_core.utils import (
    StrEnumMeta,
)


class GatewayNetworkStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    CREATED = "created"
    ATTACHING = "attaching"
    CONFIGURING = "configuring"
    READY = "ready"
    DETACHING = "detaching"

    def __str__(self) -> str:
        return str(self.value)


class GatewayStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    STOPPED = "stopped"
    ALLOCATING = "allocating"
    CONFIGURING = "configuring"
    RUNNING = "running"
    STOPPING = "stopping"
    FAILED = "failed"
    DELETING = "deleting"
    LOCKED = "locked"

    def __str__(self) -> str:
        return str(self.value)


class ListGatewayNetworksRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    STATUS_ASC = "status_asc"
    STATUS_DESC = "status_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListGatewaysRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    TYPE_ASC = "type_asc"
    TYPE_DESC = "type_desc"
    STATUS_ASC = "status_asc"
    STATUS_DESC = "status_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListIPsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    ADDRESS_ASC = "address_asc"
    ADDRESS_DESC = "address_desc"
    REVERSE_ASC = "reverse_asc"
    REVERSE_DESC = "reverse_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListPatRulesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    PUBLIC_PORT_ASC = "public_port_asc"
    PUBLIC_PORT_DESC = "public_port_desc"

    def __str__(self) -> str:
        return str(self.value)


class PatRuleProtocol(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_PROTOCOL = "unknown_protocol"
    BOTH = "both"
    TCP = "tcp"
    UDP = "udp"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class GatewayNetwork:
    id: str
    """
    ID of the Public Gateway-Private Network connection.
    """

    gateway_id: str
    """
    ID of the connected Public Gateway.
    """

    private_network_id: str
    """
    ID of the connected Private Network.
    """

    masquerade_enabled: bool
    """
    Defines whether the gateway masquerades traffic for this Private Network (Dynamic NAT).
    """

    status: GatewayNetworkStatus
    """
    Current status of the Public Gateway's connection to the Private Network.
    """

    push_default_route: bool
    """
    Enabling the default route also enables masquerading.
    """

    ipam_ip_id: str
    """
    Use this IPAM-booked IP ID as the Gateway's IP in this Private Network.
    """

    zone: ScwZone
    """
    Zone of the GatewayNetwork connection.
    """

    created_at: Optional[datetime]
    """
    Connection creation date.
    """

    updated_at: Optional[datetime]
    """
    Connection last modification date.
    """

    mac_address: Optional[str]
    """
    MAC address of the gateway in the Private Network (if the gateway is up and running).
    """


@dataclass
class IP:
    id: str
    """
    IP address ID.
    """

    organization_id: str
    """
    Owning Organization.
    """

    project_id: str
    """
    Owning Project.
    """

    tags: List[str]
    """
    Tags associated with the IP address.
    """

    address: str
    """
    The IP address itself.
    """

    zone: ScwZone
    """
    Zone of the IP address.
    """

    created_at: Optional[datetime]
    """
    IP address creation date.
    """

    updated_at: Optional[datetime]
    """
    IP address last modification date.
    """

    reverse: Optional[str]
    """
    Reverse domain name for the IP address.
    """

    gateway_id: Optional[str]
    """
    Public Gateway associated with the IP address.
    """


@dataclass
class GatewayType:
    name: str
    """
    Public Gateway type name.
    """

    bandwidth: int
    """
    Bandwidth, in bps, of the Public Gateway. This is the public bandwidth to the outer Internet, and the internal bandwidth to each connected Private Networks.
    """

    zone: ScwZone
    """
    Zone the Public Gateway type is available in.
    """


@dataclass
class Gateway:
    id: str
    """
    ID of the gateway.
    """

    organization_id: str
    """
    Owning Organization.
    """

    project_id: str
    """
    Owning Project.
    """

    type_: str
    """
    Gateway type name (commercial offer).
    """

    bandwidth: int
    """
    Bandwidth available of the gateway.
    """

    status: GatewayStatus
    """
    Current status of the gateway.
    """

    name: str
    """
    Name of the gateway.
    """

    created_at: Optional[datetime]
    """
    Gateway creation date.
    """

    updated_at: Optional[datetime]
    """
    Gateway last modification date.
    """

    tags: List[str]
    """
    Tags associated with the gateway.
    """

    gateway_networks: List[GatewayNetwork]
    """
    GatewayNetwork objects attached to the gateway (each one represents a connection to a Private Network).
    """

    bastion_enabled: bool
    """
    Defines whether SSH bastion is enabled on the gateway.
    """

    ipv4: Optional[IP]
    """
    Public IPv4 address of the gateway.
    """

    version: Optional[str]
    """
    Version of the running gateway software.
    """

    can_upgrade_to: Optional[str]
    """
    Newly available gateway software version that can be updated to.
    """

    bastion_port: int
    """
    Port of the SSH bastion.
    """

    smtp_enabled: bool
    """
    Defines whether SMTP traffic is allowed to pass through the gateway.
    """

    is_legacy: bool
    """
    Defines whether the gateway uses non-IPAM IP configurations.
    """

    bastion_allowed_ips: List[str]
    """
    Ranges of IP addresses allowed to connect to the gateway's SSH bastion.
    """

    zone: ScwZone
    """
    Zone of the gateway.
    """


@dataclass
class PatRule:
    id: str
    """
    PAT rule ID.
    """

    gateway_id: str
    """
    Gateway the PAT rule applies to.
    """

    public_port: int
    """
    Public port to listen on.
    """

    private_ip: str
    """
    Private IP address to forward data to.
    """

    private_port: int
    """
    Private port to translate to.
    """

    protocol: PatRuleProtocol
    """
    Protocol the rule applies to.
    """

    zone: ScwZone
    """
    Zone of the PAT rule.
    """

    created_at: Optional[datetime]
    """
    PAT rule creation date.
    """

    updated_at: Optional[datetime]
    """
    PAT rule last modification date.
    """


@dataclass
class SetPatRulesRequestRule:
    public_port: int
    """
    Public port to listen on. Uniquely identifies the rule, and a matching rule will be updated with the new parameters.
    """

    private_ip: str
    """
    Private IP to forward data to.
    """

    private_port: int
    """
    Private port to translate to.
    """

    protocol: PatRuleProtocol
    """
    Protocol the rule should apply to.
    """


@dataclass
class AddBastionAllowedIPsRequest:
    gateway_id: str
    """
    ID of the gateway to add the allowed IP range to.
    """

    ip_range: str
    """
    IP range allowed to connect to the SSH bastion.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class AddBastionAllowedIPsResponse:
    ip_ranges: List[str]
    """
    Ranges of IP addresses allowed to connect to the gateway's SSH bastion.
    """


@dataclass
class CreateGatewayNetworkRequest:
    gateway_id: str
    """
    Public Gateway to connect.
    """

    private_network_id: str
    """
    Private Network to connect.
    """

    enable_masquerade: bool
    """
    Defines whether to enable masquerade (dynamic NAT) on the GatewayNetwork.
    """

    push_default_route: bool
    """
    Enabling the default route also enables masquerading.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    ipam_ip_id: Optional[str]
    """
    Use this IPAM-booked IP ID as the Gateway's IP in this Private Network.
    """


@dataclass
class CreateGatewayRequest:
    type_: str
    """
    Gateway type (commercial offer type).
    """

    enable_smtp: bool
    """
    Defines whether SMTP traffic should be allowed pass through the gateway.
    """

    enable_bastion: bool
    """
    Defines whether SSH bastion should be enabled the gateway.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    project_id: Optional[str]
    """
    Scaleway Project to create the gateway in.
    """

    name: Optional[str]
    """
    Name for the gateway.
    """

    tags: Optional[List[str]]
    """
    Tags for the gateway.
    """

    ip_id: Optional[str]
    """
    Existing IP address to attach to the gateway.
    """

    bastion_port: Optional[int]
    """
    Port of the SSH bastion.
    """


@dataclass
class CreateIPRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    project_id: Optional[str]
    """
    Project to create the IP address in.
    """

    tags: Optional[List[str]]
    """
    Tags to give to the IP address.
    """


@dataclass
class CreatePatRuleRequest:
    gateway_id: str
    """
    ID of the Gateway on which to create the rule.
    """

    public_port: int
    """
    Public port to listen on.
    """

    private_ip: str
    """
    Private IP to forward data to.
    """

    private_port: int
    """
    Private port to translate to.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    protocol: Optional[PatRuleProtocol]
    """
    Protocol the rule should apply to.
    """


@dataclass
class DeleteBastionAllowedIPsRequest:
    gateway_id: str
    """
    ID of the gateway on which to delete the allowed IP range.
    """

    ip_range: str
    """
    IP range to delete from SSH bastion's list of allowed IPs.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class DeleteGatewayNetworkRequest:
    gateway_network_id: str
    """
    ID of the GatewayNetwork to delete.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class DeleteGatewayRequest:
    gateway_id: str
    """
    ID of the gateway to delete.
    """

    delete_ip: bool
    """
    Defines whether the PGW's IP should be deleted.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class DeleteIPRequest:
    ip_id: str
    """
    ID of the IP address to delete.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class DeletePatRuleRequest:
    pat_rule_id: str
    """
    ID of the PAT rule to delete.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetGatewayNetworkRequest:
    gateway_network_id: str
    """
    ID of the GatewayNetwork to fetch.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetGatewayRequest:
    gateway_id: str
    """
    ID of the gateway to fetch.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetIPRequest:
    ip_id: str
    """
    ID of the IP address to get.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetPatRuleRequest:
    pat_rule_id: str
    """
    ID of the PAT rule to get.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ListGatewayNetworksRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListGatewayNetworksRequestOrderBy]
    """
    Order in which to return results.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    GatewayNetworks per page.
    """

    status: Optional[List[GatewayNetworkStatus]]
    """
    Filter for GatewayNetworks with these status. Use `unknown` to include all statuses.
    """

    gateway_ids: Optional[List[str]]
    """
    Filter for GatewayNetworks connected to these gateways.
    """

    private_network_ids: Optional[List[str]]
    """
    Filter for GatewayNetworks connected to these Private Networks.
    """

    masquerade_enabled: Optional[bool]
    """
    Filter for GatewayNetworks with this `enable_masquerade` setting.
    """


@dataclass
class ListGatewayNetworksResponse:
    gateway_networks: List[GatewayNetwork]
    """
    GatewayNetworks on this page.
    """

    total_count: int
    """
    Total GatewayNetworks count matching the filter.
    """


@dataclass
class ListGatewayTypesRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ListGatewayTypesResponse:
    types: List[GatewayType]
    """
    Available types of Public Gateway.
    """


@dataclass
class ListGatewaysRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListGatewaysRequestOrderBy]
    """
    Order in which to return results.
    """

    page: Optional[int]
    """
    Page number to return.
    """

    page_size: Optional[int]
    """
    Gateways per page.
    """

    organization_id: Optional[str]
    """
    Include only gateways in this Organization.
    """

    project_id: Optional[str]
    """
    Include only gateways in this Project.
    """

    name: Optional[str]
    """
    Filter for gateways which have this search term in their name.
    """

    tags: Optional[List[str]]
    """
    Filter for gateways with these tags.
    """

    types: Optional[List[str]]
    """
    Filter for gateways of these types.
    """

    status: Optional[List[GatewayStatus]]
    """
    Filter for gateways with these status. Use `unknown` to include all statuses.
    """

    private_network_ids: Optional[List[str]]
    """
    Filter for gateways attached to these Private Networks.
    """

    include_legacy: Optional[bool]
    """
    Include also legacy gateways.
    """


@dataclass
class ListGatewaysResponse:
    gateways: List[Gateway]
    """
    Gateways on this page.
    """

    total_count: int
    """
    Total count of gateways matching the filter.
    """


@dataclass
class ListIPsRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListIPsRequestOrderBy]
    """
    Order in which to return results.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    IP addresses per page.
    """

    organization_id: Optional[str]
    """
    Include only gateways in this Organization.
    """

    project_id: Optional[str]
    """
    Filter for IP addresses in this Project.
    """

    tags: Optional[List[str]]
    """
    Filter for IP addresses with these tags.
    """

    reverse: Optional[str]
    """
    Filter for IP addresses that have a reverse containing this string.
    """

    is_free: Optional[bool]
    """
    Filter based on whether the IP is attached to a gateway or not.
    """


@dataclass
class ListIPsResponse:
    ips: List[IP]
    """
    IP addresses on this page.
    """

    total_count: int
    """
    Total count of IP addresses matching the filter.
    """


@dataclass
class ListPatRulesRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListPatRulesRequestOrderBy]
    """
    Order in which to return results.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    PAT rules per page.
    """

    gateway_ids: Optional[List[str]]
    """
    Filter for PAT rules on these gateways.
    """

    private_ips: Optional[List[str]]
    """
    Filter for PAT rules targeting these private ips.
    """

    protocol: Optional[PatRuleProtocol]
    """
    Filter for PAT rules with this protocol.
    """


@dataclass
class ListPatRulesResponse:
    pat_rules: List[PatRule]
    """
    Array of PAT rules matching the filter.
    """

    total_count: int
    """
    Total count of PAT rules matching the filter.
    """


@dataclass
class RefreshSSHKeysRequest:
    gateway_id: str
    """
    ID of the gateway to refresh SSH keys on.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class SetBastionAllowedIPsRequest:
    gateway_id: str
    """
    ID of the gateway on which to set the allowed IP range.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    ip_ranges: Optional[List[str]]
    """
    New list of IP ranges (each range in CIDR notation) allowed to connect to the SSH bastion.
    """


@dataclass
class SetBastionAllowedIPsResponse:
    ip_ranges: List[str]
    """
    Ranges of IP addresses allowed to connect to the gateway's SSH bastion.
    """


@dataclass
class SetPatRulesRequest:
    gateway_id: str
    """
    ID of the gateway on which to set the PAT rules.
    """

    pat_rules: List[SetPatRulesRequestRule]
    """
    New list of PAT rules.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class SetPatRulesResponse:
    pat_rules: List[PatRule]
    """
    List of PAT rules.
    """


@dataclass
class UpdateGatewayNetworkRequest:
    gateway_network_id: str
    """
    ID of the GatewayNetwork to update.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    enable_masquerade: Optional[bool]
    """
    Defines whether to enable masquerade (dynamic NAT) on the GatewayNetwork.
    """

    push_default_route: Optional[bool]
    """
    Enabling the default route also enables masquerading.
    """

    ipam_ip_id: Optional[str]
    """
    Use this IPAM-booked IP ID as the Gateway's IP in this Private Network.
    """


@dataclass
class UpdateGatewayRequest:
    gateway_id: str
    """
    ID of the gateway to update.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    name: Optional[str]
    """
    Name for the gateway.
    """

    tags: Optional[List[str]]
    """
    Tags for the gateway.
    """

    enable_bastion: Optional[bool]
    """
    Defines whether SSH bastion should be enabled the gateway.
    """

    bastion_port: Optional[int]
    """
    Port of the SSH bastion.
    """

    enable_smtp: Optional[bool]
    """
    Defines whether SMTP traffic should be allowed to pass through the gateway.
    """


@dataclass
class UpdateIPRequest:
    ip_id: str
    """
    ID of the IP address to update.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    tags: Optional[List[str]]
    """
    Tags to give to the IP address.
    """

    reverse: Optional[str]
    """
    Reverse to set on the address. Empty string to unset.
    """

    gateway_id: Optional[str]
    """
    Gateway to attach the IP address to. Empty string to detach.
    """


@dataclass
class UpdatePatRuleRequest:
    pat_rule_id: str
    """
    ID of the PAT rule to update.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    public_port: Optional[int]
    """
    Public port to listen on.
    """

    private_ip: Optional[str]
    """
    Private IP to forward data to.
    """

    private_port: Optional[int]
    """
    Private port to translate to.
    """

    protocol: Optional[PatRuleProtocol]
    """
    Protocol the rule should apply to.
    """


@dataclass
class UpgradeGatewayRequest:
    gateway_id: str
    """
    ID of the gateway to upgrade.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    type_: Optional[str]
    """
    Gateway type (commercial offer).
    """



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict
from dateutil import parser

from scaleway_core.profile import ProfileDefaults
from .types import (
    Name,
    PinCIDMeta,
    PinCID,
    PinInfo,
    Pin,
    Volume,
    ExportKeyNameResponse,
    ListNamesResponse,
    ListPinsResponse,
    ListVolumesResponse,
    ReplacePinResponse,
    CreatePinByCIDRequest,
    CreatePinByURLRequest,
    CreateVolumeRequest,
    IpnsApiCreateNameRequest,
    IpnsApiImportKeyNameRequest,
    IpnsApiUpdateNameRequest,
    ReplacePinRequest,
    UpdateVolumeRequest,
)


def unmarshal_Name(data: Any) -> Name:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Name' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("name_id", None)
    if field is not None:
        args["name_id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("tags", None)
    if field is not None:
        args["tags"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("key", None)
    if field is not None:
        args["key"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("value", None)
    if field is not None:
        args["value"] = field

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field

    return Name(**args)


def unmarshal_PinCIDMeta(data: Any) -> PinCIDMeta:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'PinCIDMeta' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    return PinCIDMeta(**args)


def unmarshal_PinCID(data: Any) -> PinCID:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'PinCID' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("origins", None)
    if field is not None:
        args["origins"] = field

    field = data.get("cid", None)
    if field is not None:
        args["cid"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("meta", None)
    if field is not None:
        args["meta"] = unmarshal_PinCIDMeta(field)

    return PinCID(**args)


def unmarshal_PinInfo(data: Any) -> PinInfo:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'PinInfo' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("status_details", None)
    if field is not None:
        args["status_details"] = field

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("url", None)
    if field is not None:
        args["url"] = field

    field = data.get("size", None)
    if field is not None:
        args["size"] = field

    field = data.get("progress", None)
    if field is not None:
        args["progress"] = field

    return PinInfo(**args)


def unmarshal_Pin(data: Any) -> Pin:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Pin' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("pin_id", None)
    if field is not None:
        args["pin_id"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("delegates", None)
    if field is not None:
        args["delegates"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field

    field = data.get("cid", None)
    if field is not None:
        args["cid"] = unmarshal_PinCID(field)

    field = data.get("info", None)
    if field is not None:
        args["info"] = unmarshal_PinInfo(field)

    return Pin(**args)


def unmarshal_Volume(data: Any) -> Volume:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Volume' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    field = data.get("count_pin", None)
    if field is not None:
        args["count_pin"] = field

    field = data.get("tags", None)
    if field is not None:
        args["tags"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field

    field = data.get("size", None)
    if field is not None:
        args["size"] = field

    return Volume(**args)


def unmarshal_ExportKeyNameResponse(data: Any) -> ExportKeyNameResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ExportKeyNameResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("name_id", None)
    if field is not None:
        args["name_id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("public_key", None)
    if field is not None:
        args["public_key"] = field

    field = data.get("private_key", None)
    if field is not None:
        args["private_key"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field

    return ExportKeyNameResponse(**args)


def unmarshal_ListNamesResponse(data: Any) -> ListNamesResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListNamesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("names", None)
    if field is not None:
        args["names"] = (
            [unmarshal_Name(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListNamesResponse(**args)


def unmarshal_ListPinsResponse(data: Any) -> ListPinsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListPinsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("pins", None)
    if field is not None:
        args["pins"] = [unmarshal_Pin(v) for v in field] if field is not None else None

    return ListPinsResponse(**args)


def unmarshal_ListVolumesResponse(data: Any) -> ListVolumesResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListVolumesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("volumes", None)
    if field is not None:
        args["volumes"] = (
            [unmarshal_Volume(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListVolumesResponse(**args)


def unmarshal_ReplacePinResponse(data: Any) -> ReplacePinResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ReplacePinResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("pin", None)
    if field is not None:
        args["pin"] = unmarshal_Pin(field)

    return ReplacePinResponse(**args)


def marshal_CreatePinByCIDRequest(
    request: CreatePinByCIDRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.volume_id is not None:
        output["volume_id"] = request.volume_id

    if request.cid is not None:
        output["cid"] = request.cid

    if request.origins is not None:
        output["origins"] = request.origins

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_CreatePinByURLRequest(
    request: CreatePinByURLRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.volume_id is not None:
        output["volume_id"] = request.volume_id

    if request.url is not None:
        output["url"] = request.url

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_CreateVolumeRequest(
    request: CreateVolumeRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_IpnsApiCreateNameRequest(
    request: IpnsApiCreateNameRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.value is not None:
        output["value"] = request.value

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_IpnsApiImportKeyNameRequest(
    request: IpnsApiImportKeyNameRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.private_key is not None:
        output["private_key"] = request.private_key

    if request.value is not None:
        output["value"] = request.value

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_IpnsApiUpdateNameRequest(
    request: IpnsApiUpdateNameRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.tags is not None:
        output["tags"] = request.tags

    if request.value is not None:
        output["value"] = request.value

    return output


def marshal_ReplacePinRequest(
    request: ReplacePinRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.volume_id is not None:
        output["volume_id"] = request.volume_id

    if request.cid is not None:
        output["cid"] = request.cid

    if request.name is not None:
        output["name"] = request.name

    if request.origins is not None:
        output["origins"] = request.origins

    return output


def marshal_UpdateVolumeRequest(
    request: UpdateVolumeRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.tags is not None:
        output["tags"] = request.tags

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import List, Optional

from scaleway_core.api import API
from scaleway_core.bridge import (
    Region as ScwRegion,
)
from scaleway_core.utils import (
    validate_path_param,
    fetch_all_pages,
)
from .types import (
    DataKeyAlgorithmSymmetricEncryption,
    KeyOrigin,
    ListKeysRequestOrderBy,
    CreateKeyRequest,
    DataKey,
    DecryptRequest,
    DecryptResponse,
    EncryptRequest,
    EncryptResponse,
    GenerateDataKeyRequest,
    ImportKeyMaterialRequest,
    Key,
    KeyRotationPolicy,
    KeyUsage,
    ListKeysResponse,
    UpdateKeyRequest,
)
from .marshalling import (
    unmarshal_Key,
    unmarshal_DataKey,
    unmarshal_DecryptResponse,
    unmarshal_EncryptResponse,
    unmarshal_ListKeysResponse,
    marshal_CreateKeyRequest,
    marshal_DecryptRequest,
    marshal_EncryptRequest,
    marshal_GenerateDataKeyRequest,
    marshal_ImportKeyMaterialRequest,
    marshal_UpdateKeyRequest,
)


class KeyManagerV1Alpha1API(API):
    """
    This API allows you to create, manage and use cryptographic keys in a centralized and secure service.
    """

    def create_key(
        self,
        *,
        unprotected: bool,
        region: Optional[ScwRegion] = None,
        project_id: Optional[str] = None,
        name: Optional[str] = None,
        usage: Optional[KeyUsage] = None,
        description: Optional[str] = None,
        tags: Optional[List[str]] = None,
        rotation_policy: Optional[KeyRotationPolicy] = None,
        origin: Optional[KeyOrigin] = None,
    ) -> Key:
        """
        Create a key.
        Create a key in a given region specified by the `region` parameter. Keys only support symmetric encryption. You can use keys to encrypt or decrypt arbitrary payloads, or to generate data encryption keys. **Data encryption keys are not stored in Key Manager**.
        :param unprotected: Default value is `false`.
        :param region: Region to target. If none is passed will use default region from the config.
        :param project_id: ID of the Project containing the key.
        :param name: (Optional) Name of the key.
        :param usage: See the `Key.Algorithm.SymmetricEncryption` enum for a description of values.
        :param description: (Optional) Description of the key.
        :param tags: (Optional) List of the key's tags.
        :param rotation_policy: If not specified, no rotation policy will be applied to the key.
        :param origin: Refer to the `Key.Origin` enum for a description of values.
        :return: :class:`Key <Key>`

        Usage:
        ::

            result = api.create_key(
                unprotected=False,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "POST",
            f"/key-manager/v1alpha1/regions/{param_region}/keys",
            body=marshal_CreateKeyRequest(
                CreateKeyRequest(
                    unprotected=unprotected,
                    region=region,
                    project_id=project_id,
                    name=name,
                    usage=usage,
                    description=description,
                    tags=tags,
                    rotation_policy=rotation_policy,
                    origin=origin,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Key(res.json())

    def get_key(
        self,
        *,
        key_id: str,
        region: Optional[ScwRegion] = None,
    ) -> Key:
        """
        Get key metadata.
        Retrieve metadata for a specified key using the `region` and `key_id` parameters.
        :param key_id: ID of the key to target.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`Key <Key>`

        Usage:
        ::

            result = api.get_key(
                key_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_key_id = validate_path_param("key_id", key_id)

        res = self._request(
            "GET",
            f"/key-manager/v1alpha1/regions/{param_region}/keys/{param_key_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Key(res.json())

    def update_key(
        self,
        *,
        key_id: str,
        region: Optional[ScwRegion] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
        tags: Optional[List[str]] = None,
        rotation_policy: Optional[KeyRotationPolicy] = None,
    ) -> Key:
        """
        Update a key.
        Modify a key's metadata including name, description and tags, specified by the `key_id` and `region` parameters.
        :param key_id: ID of the key to update.
        :param region: Region to target. If none is passed will use default region from the config.
        :param name: (Optional) Updated name of the key.
        :param description: (Optional) Updated description of the key.
        :param tags: (Optional) Updated list of the key's tags.
        :param rotation_policy: If not specified, the key's existing rotation policy applies.
        :return: :class:`Key <Key>`

        Usage:
        ::

            result = api.update_key(
                key_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_key_id = validate_path_param("key_id", key_id)

        res = self._request(
            "PATCH",
            f"/key-manager/v1alpha1/regions/{param_region}/keys/{param_key_id}",
            body=marshal_UpdateKeyRequest(
                UpdateKeyRequest(
                    key_id=key_id,
                    region=region,
                    name=name,
                    description=description,
                    tags=tags,
                    rotation_policy=rotation_policy,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Key(res.json())

    def delete_key(
        self,
        *,
        key_id: str,
        region: Optional[ScwRegion] = None,
    ) -> None:
        """
        Delete a key.
        Permanently delete a key specified by the `region` and `key_id` parameters. This action is irreversible. Any data encrypted with this key, including data encryption keys, will no longer be decipherable.
        :param key_id: ID of the key to delete.
        :param region: Region to target. If none is passed will use default region from the config.

        Usage:
        ::

            result = api.delete_key(
                key_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_key_id = validate_path_param("key_id", key_id)

        res = self._request(
            "DELETE",
            f"/key-manager/v1alpha1/regions/{param_region}/keys/{param_key_id}",
        )

        self._throw_on_error(res)

    def rotate_key(
        self,
        *,
        key_id: str,
        region: Optional[ScwRegion] = None,
    ) -> Key:
        """
        Rotate a key.
        Generate a new version of an existing key with new key material. Previous key versions remain usable to decrypt previously encrypted data, but the key's new version will be used for subsequent encryption operations and data key generation.
        :param key_id: ID of the key to rotate.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`Key <Key>`

        Usage:
        ::

            result = api.rotate_key(
                key_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_key_id = validate_path_param("key_id", key_id)

        res = self._request(
            "POST",
            f"/key-manager/v1alpha1/regions/{param_region}/keys/{param_key_id}/rotate",
            body={},
        )

        self._throw_on_error(res)
        return unmarshal_Key(res.json())

    def protect_key(
        self,
        *,
        key_id: str,
        region: Optional[ScwRegion] = None,
    ) -> Key:
        """
        Apply key protection.
        Apply protection to a given key specified by the `key_id` parameter. Applying key protection means that your key can be used and modified, but it cannot be deleted.
        :param key_id: ID of the key to apply key protection to.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`Key <Key>`

        Usage:
        ::

            result = api.protect_key(
                key_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_key_id = validate_path_param("key_id", key_id)

        res = self._request(
            "POST",
            f"/key-manager/v1alpha1/regions/{param_region}/keys/{param_key_id}/protect",
            body={},
        )

        self._throw_on_error(res)
        return unmarshal_Key(res.json())

    def unprotect_key(
        self,
        *,
        key_id: str,
        region: Optional[ScwRegion] = None,
    ) -> Key:
        """
        Remove key protection.
        Remove key protection from a given key specified by the `key_id` parameter. Removing key protection means that your key can be deleted anytime.
        :param key_id: ID of the key to remove key protection from.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`Key <Key>`

        Usage:
        ::

            result = api.unprotect_key(
                key_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_key_id = validate_path_param("key_id", key_id)

        res = self._request(
            "POST",
            f"/key-manager/v1alpha1/regions/{param_region}/keys/{param_key_id}/unprotect",
            body={},
        )

        self._throw_on_error(res)
        return unmarshal_Key(res.json())

    def enable_key(
        self,
        *,
        key_id: str,
        region: Optional[ScwRegion] = None,
    ) -> Key:
        """
        Enable key.
        Enable a given key to be used for cryptographic operations. Enabling a key allows you to make a disabled key usable again. You must specify the `region` and `key_id` parameters.
        :param key_id: ID of the key to enable.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`Key <Key>`

        Usage:
        ::

            result = api.enable_key(
                key_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_key_id = validate_path_param("key_id", key_id)

        res = self._request(
            "POST",
            f"/key-manager/v1alpha1/regions/{param_region}/keys/{param_key_id}/enable",
            body={},
        )

        self._throw_on_error(res)
        return unmarshal_Key(res.json())

    def disable_key(
        self,
        *,
        key_id: str,
        region: Optional[ScwRegion] = None,
    ) -> Key:
        """
        Disable key.
        Disable a given key, preventing it to be used for cryptographic operations. Disabling a key renders it unusable. You must specify the `region` and `key_id` parameters.
        :param key_id: ID of the key to disable.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`Key <Key>`

        Usage:
        ::

            result = api.disable_key(
                key_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_key_id = validate_path_param("key_id", key_id)

        res = self._request(
            "POST",
            f"/key-manager/v1alpha1/regions/{param_region}/keys/{param_key_id}/disable",
            body={},
        )

        self._throw_on_error(res)
        return unmarshal_Key(res.json())

    def list_keys(
        self,
        *,
        region: Optional[ScwRegion] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        order_by: Optional[ListKeysRequestOrderBy] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        tags: Optional[List[str]] = None,
        name: Optional[str] = None,
    ) -> ListKeysResponse:
        """
        List keys.
        Retrieve a list of keys across all Projects in an Organization or within a specific Project. You must specify the `region`, and either the `organization_id` or the `project_id`.
        :param region: Region to target. If none is passed will use default region from the config.
        :param organization_id: (Optional) Filter by Organization ID.
        :param project_id: (Optional) Filter by Project ID.
        :param order_by:
        :param page:
        :param page_size:
        :param tags: (Optional) List of tags to filter on.
        :param name: (Optional) Filter by key name.
        :return: :class:`ListKeysResponse <ListKeysResponse>`

        Usage:
        ::

            result = api.list_keys()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/key-manager/v1alpha1/regions/{param_region}/keys",
            params={
                "name": name,
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "project_id": project_id or self.client.default_project_id,
                "tags": tags,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListKeysResponse(res.json())

    def list_keys_all(
        self,
        *,
        region: Optional[ScwRegion] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        order_by: Optional[ListKeysRequestOrderBy] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        tags: Optional[List[str]] = None,
        name: Optional[str] = None,
    ) -> List[Key]:
        """
        List keys.
        Retrieve a list of keys across all Projects in an Organization or within a specific Project. You must specify the `region`, and either the `organization_id` or the `project_id`.
        :param region: Region to target. If none is passed will use default region from the config.
        :param organization_id: (Optional) Filter by Organization ID.
        :param project_id: (Optional) Filter by Project ID.
        :param order_by:
        :param page:
        :param page_size:
        :param tags: (Optional) List of tags to filter on.
        :param name: (Optional) Filter by key name.
        :return: :class:`List[Key] <List[Key]>`

        Usage:
        ::

            result = api.list_keys_all()
        """

        return fetch_all_pages(
            type=ListKeysResponse,
            key="keys",
            fetcher=self.list_keys,
            args={
                "region": region,
                "organization_id": organization_id,
                "project_id": project_id,
                "order_by": order_by,
                "page": page,
                "page_size": page_size,
                "tags": tags,
                "name": name,
            },
        )

    def generate_data_key(
        self,
        *,
        key_id: str,
        without_plaintext: bool,
        region: Optional[ScwRegion] = None,
        algorithm: Optional[DataKeyAlgorithmSymmetricEncryption] = None,
    ) -> DataKey:
        """
        Create a data encryption key.
        Create a new data encryption key for cryptographic operations outside of Key Manager. The data encryption key is encrypted and must be decrypted using the key you have created in Key Manager.

        The data encryption key is returned in plaintext and ciphertext but it should only be stored in its encrypted form (ciphertext). Key Manager does not store your data encryption key. To retrieve your key's plaintext, use the `Decrypt` method with your key's ID and ciphertext.
        :param key_id: ID of the key.
        :param without_plaintext: Default value is `false`, meaning that the plaintext is returned.
        Set it to `true` if you do not wish the plaintext to be returned in the response object.
        :param region: Region to target. If none is passed will use default region from the config.
        :param algorithm: See the `DataKey.Algorithm.SymmetricEncryption` enum for a description of values.
        :return: :class:`DataKey <DataKey>`

        Usage:
        ::

            result = api.generate_data_key(
                key_id="example",
                without_plaintext=False,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_key_id = validate_path_param("key_id", key_id)

        res = self._request(
            "POST",
            f"/key-manager/v1alpha1/regions/{param_region}/keys/{param_key_id}/generate-data-key",
            body=marshal_GenerateDataKeyRequest(
                GenerateDataKeyRequest(
                    key_id=key_id,
                    without_plaintext=without_plaintext,
                    region=region,
                    algorithm=algorithm,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_DataKey(res.json())

    def encrypt(
        self,
        *,
        key_id: str,
        plaintext: str,
        region: Optional[ScwRegion] = None,
        associated_data: Optional[str] = None,
    ) -> EncryptResponse:
        """
        Encrypt a payload.
        Encrypt a payload using an existing key, specified by the `key_id` parameter. Only keys with a usage set to `symmetric_encryption` are supported by this method. The maximum payload size that can be encrypted is 64 KB of plaintext.
        :param key_id: ID of the key to encrypt.
        :param plaintext: Data size must be between 1 and 65535 bytes.
        :param region: Region to target. If none is passed will use default region from the config.
        :param associated_data: Additional data which will not be encrypted, but authenticated and appended to the encrypted payload.
        :return: :class:`EncryptResponse <EncryptResponse>`

        Usage:
        ::

            result = api.encrypt(
                key_id="example",
                plaintext="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_key_id = validate_path_param("key_id", key_id)

        res = self._request(
            "POST",
            f"/key-manager/v1alpha1/regions/{param_region}/keys/{param_key_id}/encrypt",
            body=marshal_EncryptRequest(
                EncryptRequest(
                    key_id=key_id,
                    plaintext=plaintext,
                    region=region,
                    associated_data=associated_data,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_EncryptResponse(res.json())

    def decrypt(
        self,
        *,
        key_id: str,
        ciphertext: str,
        region: Optional[ScwRegion] = None,
        associated_data: Optional[str] = None,
    ) -> DecryptResponse:
        """
        Decrypt an encrypted payload.
        Decrypt an encrypted payload using an existing key, specified by the `key_id` parameter. The maximum payload size that can be decrypted is equivalent to the encrypted output of 64 KB of data (around 131 KB).
        :param key_id: ID of the key to decrypt.
        :param ciphertext: Data size must be between 1 and 131071 bytes.
        :param region: Region to target. If none is passed will use default region from the config.
        :param associated_data: The additional data must match the value passed in the encryption request.
        :return: :class:`DecryptResponse <DecryptResponse>`

        Usage:
        ::

            result = api.decrypt(
                key_id="example",
                ciphertext="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_key_id = validate_path_param("key_id", key_id)

        res = self._request(
            "POST",
            f"/key-manager/v1alpha1/regions/{param_region}/keys/{param_key_id}/decrypt",
            body=marshal_DecryptRequest(
                DecryptRequest(
                    key_id=key_id,
                    ciphertext=ciphertext,
                    region=region,
                    associated_data=associated_data,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_DecryptResponse(res.json())

    def import_key_material(
        self,
        *,
        key_id: str,
        key_material: str,
        region: Optional[ScwRegion] = None,
        salt: Optional[str] = None,
    ) -> Key:
        """
        Import key material.
        Import externally generated key material into Key Manager to derive a new cryptographic key. The key's origin must be `external`.
        :param key_id: The key's origin must be `external`.
        :param key_material: The key material The key material is a random sequence of bytes used to derive a cryptographic key.
        :param region: Region to target. If none is passed will use default region from the config.
        :param salt: A salt is random data added to key material to ensure unique derived keys, even if the input is similar. It helps strengthen security when the key material has low randomness (low entropy).
        :return: :class:`Key <Key>`

        Usage:
        ::

            result = api.import_key_material(
                key_id="example",
                key_material="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_key_id = validate_path_param("key_id", key_id)

        res = self._request(
            "POST",
            f"/key-manager/v1alpha1/regions/{param_region}/keys/{param_key_id}/import-key-material",
            body=marshal_ImportKeyMaterialRequest(
                ImportKeyMaterialRequest(
                    key_id=key_id,
                    key_material=key_material,
                    region=region,
                    salt=salt,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Key(res.json())

    def delete_key_material(
        self,
        *,
        key_id: str,
        region: Optional[ScwRegion] = None,
    ) -> None:
        """
        Delete key material.
        Delete previously imported key material. This renders the associated cryptographic key unusable for any operation. The key's origin must be `external`.
        :param key_id: ID of the key of which to delete the key material.
        :param region: Region to target. If none is passed will use default region from the config.

        Usage:
        ::

            result = api.delete_key_material(
                key_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_key_id = validate_path_param("key_id", key_id)

        res = self._request(
            "POST",
            f"/key-manager/v1alpha1/regions/{param_region}/keys/{param_key_id}/delete-key-material",
            body={},
        )

        self._throw_on_error(res)



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Dict, List, Optional

from scaleway_core.bridge import (
    Money,
    Zone as ScwZone,
)
from scaleway_core.utils import (
    StrEnumMeta,
)


class AttachFailoverIPToMacAddressRequestMacType(str, Enum, metaclass=StrEnumMeta):
    MAC_TYPE_UNKNOWN = "mac_type_unknown"
    VMWARE = "vmware"
    KVM = "kvm"
    XEN = "xen"

    def __str__(self) -> str:
        return str(self.value)


class BMCAccessStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    CREATING = "creating"
    CREATED = "created"
    DELETING = "deleting"

    def __str__(self) -> str:
        return str(self.value)


class BackupStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_BACKUP_STATUS = "unknown_backup_status"
    UNINITIALIZED = "uninitialized"
    INACTIVE = "inactive"
    READY = "ready"

    def __str__(self) -> str:
        return str(self.value)


class FailoverBlockVersion(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_VERSION = "unknown_version"
    IPV4 = "ipv4"
    IPV6 = "ipv6"

    def __str__(self) -> str:
        return str(self.value)


class FailoverIPInterfaceType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    NORMAL = "normal"
    IPMI = "ipmi"
    VIRTUAL = "virtual"

    def __str__(self) -> str:
        return str(self.value)


class FailoverIPStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    READY = "ready"
    BUSY = "busy"
    LOCKED = "locked"

    def __str__(self) -> str:
        return str(self.value)


class FailoverIPVersion(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_VERSION = "unknown_version"
    IPV4 = "ipv4"
    IPV6 = "ipv6"

    def __str__(self) -> str:
        return str(self.value)


class GetRpnStatusResponseStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    BUSY = "busy"
    OPERATIONAL = "operational"

    def __str__(self) -> str:
        return str(self.value)


class IPSemantic(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    PROXAD = "proxad"
    EXT = "ext"
    PUBLIC = "public"
    PRIVATE = "private"
    IPMI = "ipmi"
    ADM = "adm"
    REDIRECT = "redirect"
    MIGRATION = "migration"

    def __str__(self) -> str:
        return str(self.value)


class IPStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    READY = "ready"
    BUSY = "busy"
    LOCKED = "locked"

    def __str__(self) -> str:
        return str(self.value)


class IPVersion(str, Enum, metaclass=StrEnumMeta):
    IPV4 = "ipv4"
    IPV6 = "ipv6"

    def __str__(self) -> str:
        return str(self.value)


class IPv6BlockDelegationStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    UPDATING = "updating"
    DONE = "done"

    def __str__(self) -> str:
        return str(self.value)


class InvoicePaymentMethod(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_PAYMENT_METHOD = "unknown_payment_method"
    CREDIT_CARD = "credit_card"
    AMEX = "amex"
    PAYPAL = "paypal"
    TRANSFER = "transfer"
    DIRECT_DEBIT = "direct_debit"

    def __str__(self) -> str:
        return str(self.value)


class InvoiceStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_INVOICE_STATUS = "unknown_invoice_status"
    UNPAID = "unpaid"
    PAID = "paid"
    ERRORED = "errored"

    def __str__(self) -> str:
        return str(self.value)


class ListFailoverIPsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    IP_ASC = "ip_asc"
    IP_DESC = "ip_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListInvoicesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListOSRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    RELEASED_AT_ASC = "released_at_asc"
    RELEASED_AT_DESC = "released_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListOffersRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    PRICE_ASC = "price_asc"
    PRICE_DESC = "price_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRefundsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRpnCapableSanServersRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRpnCapableServersRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRpnGroupMembersRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRpnGroupsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRpnInvitesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRpnSansRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRpnServerCapabilitiesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRpnV2CapableResourcesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRpnV2GroupLogsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRpnV2GroupsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRpnV2MembersRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRpnV2MembersRequestType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_TYPE = "unknown_type"
    RPNV1_GROUP = "rpnv1_group"
    SERVER = "server"

    def __str__(self) -> str:
        return str(self.value)


class ListServerDisksRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListServerEventsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListServersRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListServicesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class LogAction(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_LOG_ACTION = "unknown_log_action"
    GROUP_CREATED = "group_created"
    GROUP_DELETED = "group_deleted"
    MEMBERS_ADDED = "members_added"
    MEMBERS_DELETED = "members_deleted"
    DESCRIPTION_UPDATED = "description_updated"
    RPNV1_MEMBERS_ADDED = "rpnv1_members_added"
    RPNV1_MEMBERS_DELETED = "rpnv1_members_deleted"
    VLAN_UPDATED = "vlan_updated"
    VLAN_UPDATED_ON_ALL_SERVERS = "vlan_updated_on_all_servers"

    def __str__(self) -> str:
        return str(self.value)


class LogStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_LOG_STATUS = "unknown_log_status"
    SUCCESS = "success"
    IN_PROGRESS = "in_progress"
    ERROR = "error"

    def __str__(self) -> str:
        return str(self.value)


class MemoryType(str, Enum, metaclass=StrEnumMeta):
    DDR2 = "ddr2"
    DDR3 = "ddr3"
    DDR4 = "ddr4"
    DDR5 = "ddr5"

    def __str__(self) -> str:
        return str(self.value)


class NetworkInterfaceInterfaceType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    NORMAL = "normal"
    IPMI = "ipmi"
    VIRTUAL = "virtual"

    def __str__(self) -> str:
        return str(self.value)


class OSArch(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_ARCH = "unknown_arch"
    AMD64 = "amd64"
    X86 = "x86"
    ARM = "arm"
    ARM64 = "arm64"

    def __str__(self) -> str:
        return str(self.value)


class OSType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_TYPE = "unknown_type"
    SERVER = "server"
    VIRTU = "virtu"
    PANEL = "panel"
    DESKTOP = "desktop"
    CUSTOM = "custom"
    RESCUE = "rescue"

    def __str__(self) -> str:
        return str(self.value)


class OfferAntiDosInfoType(str, Enum, metaclass=StrEnumMeta):
    MINIMAL = "minimal"
    PREVENTIVE = "preventive"
    CURATIVE = "curative"

    def __str__(self) -> str:
        return str(self.value)


class OfferCatalog(str, Enum, metaclass=StrEnumMeta):
    ALL = "all"
    DEFAULT = "default"
    BETA = "beta"
    RESELLER = "reseller"
    PREMIUM = "premium"
    VOLUME = "volume"
    ADMIN = "admin"
    INACTIVE = "inactive"

    def __str__(self) -> str:
        return str(self.value)


class OfferPaymentFrequency(str, Enum, metaclass=StrEnumMeta):
    MONTHLY = "monthly"
    ONESHOT = "oneshot"

    def __str__(self) -> str:
        return str(self.value)


class OfferSANInfoType(str, Enum, metaclass=StrEnumMeta):
    HDD = "hdd"
    SSD = "ssd"

    def __str__(self) -> str:
        return str(self.value)


class OfferServerInfoStock(str, Enum, metaclass=StrEnumMeta):
    EMPTY = "empty"
    LOW = "low"
    AVAILABLE = "available"

    def __str__(self) -> str:
        return str(self.value)


class PartitionFileSystem(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    EFI = "efi"
    SWAP = "swap"
    EXT4 = "ext4"
    EXT3 = "ext3"
    EXT2 = "ext2"
    XFS = "xfs"
    NTFS = "ntfs"
    FAT32 = "fat32"
    UFS = "ufs"

    def __str__(self) -> str:
        return str(self.value)


class PartitionType(str, Enum, metaclass=StrEnumMeta):
    PRIMARY = "primary"
    EXTENDED = "extended"
    LOGICAL = "logical"

    def __str__(self) -> str:
        return str(self.value)


class RaidArrayRaidLevel(str, Enum, metaclass=StrEnumMeta):
    NO_RAID = "no_raid"
    RAID0 = "raid0"
    RAID1 = "raid1"
    RAID5 = "raid5"
    RAID6 = "raid6"
    RAID10 = "raid10"

    def __str__(self) -> str:
        return str(self.value)


class RefundMethod(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_REFUND_METHOD = "unknown_refund_method"
    CREDIT_CARD = "credit_card"
    AMEX = "amex"
    PAYPAL = "paypal"
    TRANSFER = "transfer"

    def __str__(self) -> str:
        return str(self.value)


class RefundStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_REFUND_STATUS = "unknown_refund_status"
    UNPAID = "unpaid"
    PAID = "paid"
    ERRORED = "errored"

    def __str__(self) -> str:
        return str(self.value)


class RescueProtocol(str, Enum, metaclass=StrEnumMeta):
    VNC = "vnc"
    SSH = "ssh"

    def __str__(self) -> str:
        return str(self.value)


class RpnGroupMemberStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_RPN_MEMBER_STATUS = "unknown_rpn_member_status"
    PENDING_INVITATION = "pending_invitation"
    ACTIVE = "active"
    CREATING = "creating"
    DELETING = "deleting"
    DELETED = "deleted"

    def __str__(self) -> str:
        return str(self.value)


class RpnGroupType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    LOCAL = "local"
    SHARED = "shared"

    def __str__(self) -> str:
        return str(self.value)


class RpnSanIpType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    SERVER_IP = "server_ip"
    RPNV2_SUBNET = "rpnv2_subnet"

    def __str__(self) -> str:
        return str(self.value)


class RpnSanStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    CREATING = "creating"
    ACTIVE = "active"
    DELETING = "deleting"
    MAINTENANCE = "maintenance"

    def __str__(self) -> str:
        return str(self.value)


class RpnV2GroupStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_GROUP_STATUS = "unknown_group_status"
    CREATING = "creating"
    ACTIVE = "active"
    UPDATING = "updating"
    DELETING = "deleting"

    def __str__(self) -> str:
        return str(self.value)


class RpnV2GroupType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_TYPE = "unknown_type"
    STANDARD = "standard"
    QINQ = "qinq"

    def __str__(self) -> str:
        return str(self.value)


class RpnV2MemberStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_MEMBER_STATUS = "unknown_member_status"
    CREATING = "creating"
    ACTIVE = "active"
    UPDATING = "updating"
    DELETING = "deleting"

    def __str__(self) -> str:
        return str(self.value)


class ServerDiskType(str, Enum, metaclass=StrEnumMeta):
    SATA = "sata"
    SSD = "ssd"
    SAS = "sas"
    SSHD = "sshd"
    USB = "usb"
    NVME = "nvme"

    def __str__(self) -> str:
        return str(self.value)


class ServerInstallStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    BOOTING = "booting"
    SETTING_UP_RAID = "setting_up_raid"
    PARTITIONING = "partitioning"
    FORMATTING = "formatting"
    INSTALLING = "installing"
    CONFIGURING = "configuring"
    CONFIGURING_BOOTLOADER = "configuring_bootloader"
    REBOOTING = "rebooting"
    INSTALLED = "installed"

    def __str__(self) -> str:
        return str(self.value)


class ServerStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    DELIVERING = "delivering"
    INSTALLING = "installing"
    READY = "ready"
    STOPPED = "stopped"
    ERROR = "error"
    LOCKED = "locked"
    RESCUE = "rescue"
    BUSY = "busy"

    def __str__(self) -> str:
        return str(self.value)


class ServiceLevelLevel(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    BASIC = "basic"
    BUSINESS = "business"

    def __str__(self) -> str:
        return str(self.value)


class ServiceProvisioningStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    DELIVERING = "delivering"
    READY = "ready"
    ERROR = "error"
    EXPIRING = "expiring"
    EXPIRED = "expired"

    def __str__(self) -> str:
        return str(self.value)


class ServiceType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_TYPE = "unknown_type"
    SERVICE = "service"
    ORDER = "order"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class OfferAntiDosInfo:
    type_: OfferAntiDosInfoType


@dataclass
class OfferBackupInfo:
    size: int


@dataclass
class OfferBandwidthInfo:
    speed: int


@dataclass
class OfferLicenseInfo:
    bound_to_ip: bool


@dataclass
class OfferRPNInfo:
    speed: int


@dataclass
class OfferSANInfo:
    size: int
    """
    SAN size (in bytes).
    """

    ha: bool
    """
    High availabilty offer.
    """

    device_type: OfferSANInfoType
    """
    Type of SAN device (hdd / ssd).
    """


@dataclass
class OfferStorageInfo:
    max_quota: int

    size: int


@dataclass
class IP:
    ip_id: int
    """
    ID of the IP.
    """

    address: str
    """
    Address of the IP.
    """

    reverse: str
    """
    Reverse IP value.
    """

    version: IPVersion
    """
    Version of IP (v4 or v6).
    """

    cidr: int
    """
    Classless InterDomain Routing notation of the IP.
    """

    netmask: str
    """
    Network mask of IP.
    """

    semantic: IPSemantic
    """
    Semantic of IP.
    """

    gateway: str
    """
    Gateway of IP.
    """

    status: IPStatus
    """
    Status of the IP.
    """


@dataclass
class Offer:
    id: int
    """
    ID of the offer.
    """

    name: str
    """
    Name of the offer.
    """

    catalog: OfferCatalog
    """
    Catalog of the offer.
    """

    payment_frequency: OfferPaymentFrequency
    """
    Payment frequency of the offer.
    """

    pricing: Optional[Money]
    """
    Price of the offer.
    """

    server_info: Optional[OfferServerInfo]

    service_level_info: Optional[OfferServiceLevelInfo]

    rpn_info: Optional[OfferRPNInfo]

    san_info: Optional[OfferSANInfo]

    antidos_info: Optional[OfferAntiDosInfo]

    backup_info: Optional[OfferBackupInfo]

    usb_storage_info: Optional[OfferStorageInfo]

    storage_info: Optional[OfferStorageInfo]

    license_info: Optional[OfferLicenseInfo]

    failover_ip_info: Optional[OfferFailoverIpInfo]

    failover_block_info: Optional[OfferFailoverBlockInfo]

    bandwidth_info: Optional[OfferBandwidthInfo]


@dataclass
class NetworkInterface:
    card_id: int
    """
    Card ID of the network interface.
    """

    device_id: int
    """
    Device ID of the network interface.
    """

    mac: str
    """
    MAC address of the network interface.
    """

    type_: NetworkInterfaceInterfaceType
    """
    Network interface type.
    """

    ips: List[IP]
    """
    IPs of the network interface.
    """


@dataclass
class OS:
    id: int
    """
    ID of the OS.
    """

    name: str
    """
    Name of the OS.
    """

    type_: OSType
    """
    Type of the OS.
    """

    version: str
    """
    Version of the OS.
    """

    arch: OSArch
    """
    Architecture of the OS.
    """

    allow_custom_partitioning: bool
    """
    True if the OS allow custom partitioning.
    """

    allow_ssh_keys: bool
    """
    True if the OS allow SSH Keys.
    """

    requires_user: bool
    """
    True if the OS requires user.
    """

    requires_admin_password: bool
    """
    True if the OS requires admin password.
    """

    requires_panel_password: bool
    """
    True if the OS requires panel password.
    """

    allowed_filesystems: List[PartitionFileSystem]
    """
    True if the OS allow file systems.
    """

    requires_license: bool
    """
    True if the OS requires license.
    """

    license_offers: List[Offer]
    """
    License offers available with the OS.
    """

    display_name: str
    """
    Display name of the OS.
    """

    password_regex: str
    """
    Regex used to validate the installation passwords.
    """

    hostname_max_length: int
    """
    Hostname max length.
    """

    max_partitions: Optional[int]
    """
    Maximum number of partitions which can be created.
    """

    panel_password_regex: Optional[str]
    """
    Regex used to validate the panel installation password.
    """

    requires_valid_hostname: Optional[bool]
    """
    If both requires_valid_hostname & hostname_regex are set, it means that at least one of the criterias must be valid.
    """

    hostname_regex: Optional[str]
    """
    If both requires_valid_hostname & hostname_regex are set, it means that at least one of the criterias must be valid.
    """

    released_at: Optional[datetime]
    """
    OS release date.
    """


@dataclass
class ServerLocation:
    rack: str

    room: str

    datacenter_name: str


@dataclass
class ServerOption:
    options: List[ServerOption]

    offer: Optional[Offer]

    created_at: Optional[datetime]

    updated_at: Optional[datetime]

    expired_at: Optional[datetime]


@dataclass
class ServiceLevel:
    offer_id: int
    """
    Offer ID of service level.
    """

    level: ServiceLevelLevel
    """
    Level type of service level.
    """


@dataclass
class RpnSan:
    id: int
    """
    RPN SAN  ID.
    """

    datacenter_name: str
    """
    Datacenter location.
    """

    organization_id: str
    """
    Organization ID.
    """

    project_id: str
    """
    Project ID.
    """

    server_hostname: str
    """
    RPN SAN server hostname.
    """

    iqn_suffix: str
    """
    IQN suffix.
    """

    offer_id: int
    """
    Offer ID.
    """

    created_at: Optional[datetime]
    """
    Date of creation of the RPN SAN.
    """

    offer_name: str
    """
    Offer description.
    """

    status: RpnSanStatus
    """
    Status.
    """

    storage_size: int
    """
    RPN SAN storage size.
    """

    iqn: str

    rpnv1_compatible: bool
    """
    True if the SAN is compatible with the RPNv1 technology.
    """

    rpnv1_implicit: bool
    """
    True if the offer supports the RPNv1 implicitly, false if it must to be added to a group to support RPNv1.
    """

    offer: Optional[Offer]

    delivered_at: Optional[datetime]
    """
    RPN SAN delivery date.
    """

    terminated_at: Optional[datetime]
    """
    RPN SAN termination date.
    """

    expires_at: Optional[datetime]
    """
    RPN SAN expiration date.
    """


@dataclass
class RpnGroup:
    id: int
    """
    Rpn group member ID.
    """

    name: str
    """
    Rpn group name.
    """

    type_: RpnGroupType
    """
    Rpn group type (local or shared).
    """

    active: bool
    """
    Whether the group is active or not.
    """

    owner: str
    """
    RPN group owner.
    """

    members_count: int
    """
    Total number of members.
    """

    organization_id: str
    """
    Rpn group organization ID.
    """

    project_id: str
    """
    Rpn group project ID.
    """

    created_at: Optional[datetime]
    """
    Rpn group creation date.
    """


@dataclass
class RpnV2GroupSubnet:
    address: str

    cidr: int


@dataclass
class Server:
    id: int
    """
    ID of the server.
    """

    organization_id: str
    """
    Organization ID the server is attached to.
    """

    project_id: str
    """
    Project ID the server is attached to.
    """

    hostname: str
    """
    Hostname of the server.
    """

    rebooted_at: Optional[datetime]
    """
    Date of last reboot of the server.
    """

    status: ServerStatus
    """
    Status of the server.
    """

    abuse_contact: str
    """
    Abuse contact of the server.
    """

    interfaces: List[NetworkInterface]
    """
    Network interfaces of the server.
    """

    zone: ScwZone
    """
    The zone in which is the server.
    """

    options: List[ServerOption]
    """
    Options subscribe on the server.
    """

    has_bmc: bool
    """
    Boolean if the server has a BMC.
    """

    tags: List[str]
    """
    Array of customs tags attached to the server.
    """

    is_outsourced: bool
    """
    Whether the server is outsourced or not.
    """

    ipv6_slaac: bool
    """
    Whether or not you can enable/disable the IPv6.
    """

    qinq: bool
    """
    Whether the server is compatible with QinQ.
    """

    is_rpnv2_member: bool
    """
    Whether or not the server is already part of an rpnv2 group.
    """

    created_at: Optional[datetime]
    """
    Date of creation of the server.
    """

    updated_at: Optional[datetime]
    """
    Date of last modification of the server.
    """

    expired_at: Optional[datetime]
    """
    Date of release of the server.
    """

    offer: Optional[Offer]
    """
    Offer of the server.
    """

    location: Optional[ServerLocation]
    """
    Location of the server.
    """

    os: Optional[OS]
    """
    OS installed on the server.
    """

    level: Optional[ServiceLevel]
    """
    Service level of the server.
    """

    rescue_os: Optional[OS]
    """
    Rescue OS of the server.
    """


@dataclass
class FailoverBlock:
    id: int
    """
    ID of the failover block.
    """

    address: str
    """
    IP of the failover block.
    """

    nameservers: List[str]
    """
    Name servers.
    """

    ip_version: FailoverBlockVersion
    """
    IP version of the failover block.
    """

    cidr: int
    """
    Classless InterDomain Routing notation of the failover block.
    """

    netmask: str
    """
    Netmask of the failover block.
    """

    gateway_ip: str
    """
    Gateway IP of the failover block.
    """


@dataclass
class RpnSanIpRpnV2Group:
    id: int

    name: str


@dataclass
class RpnSanIpServer:
    id: int

    hostname: str

    datacenter_name: str


@dataclass
class RpnSanServer:
    id: int
    """
    The RPN SAN server ID.
    """

    datacenter_name: str
    """
    The RPN SAN server datacenter name.
    """

    hostname: str
    """
    The RPN SAN server hostname.
    """

    sans: List[RpnSan]
    """
    RPN SANs linked to the RPN SAN server.
    """

    zone: ScwZone
    """
    The RPN SAN server zone.
    """


@dataclass
class RpnV2Group:
    id: int
    """
    RPN V2 group ID.
    """

    name: str
    """
    RPN V2 group name.
    """

    compatible_rpnv1: bool
    """
    Whether or not the RPN V1 compatibility was enabled.
    """

    organization_id: str
    """
    Organization ID of the RPN V2 group.
    """

    project_id: str
    """
    Project ID of the RPN V2 group.
    """

    type_: RpnV2GroupType
    """
    RPN V2 group type (qing / standard).
    """

    status: RpnV2GroupStatus
    """
    RPN V2 group status.
    """

    owner: str
    """
    RPN V2 group owner.
    """

    members_count: int
    """
    Total number of members.
    """

    gateway: str
    """
    RPN V2 gateway.
    """

    subnet: Optional[RpnV2GroupSubnet]
    """
    RPN V2 subnet.
    """

    rpnv1_group: Optional[RpnGroup]
    """
    The RPNv1 group (if the compatibility was enabled).
    """


@dataclass
class RpnV2Member:
    id: int
    """
    RPN V2 member ID.
    """

    status: RpnV2MemberStatus
    """
    RPN V2 member status.
    """

    vlan: str
    """
    RPN V2 member VLAN.
    """

    speed: Optional[int]
    """
    RPN speed.
    """

    server: Optional[Server]

    rpnv1_group: Optional[RpnGroup]


@dataclass
class ServerDisk:
    id: int

    connector: str

    type_: ServerDiskType

    capacity: int

    is_addon: bool


@dataclass
class Service:
    id: int
    """
    ID of the service.
    """

    provisioning_status: ServiceProvisioningStatus
    """
    Provisioning status of the service.
    """

    type_: ServiceType
    """
    Service type, either order or service.
    """

    resource_id: Optional[int]
    """
    Resource ID of the service.
    """

    offer: Optional[Offer]
    """
    Offer of the service.
    """

    created_at: Optional[datetime]
    """
    Creation date of the service.
    """

    delivered_at: Optional[datetime]
    """
    Delivery date of the service.
    """

    terminated_at: Optional[datetime]
    """
    Terminatation date of the service.
    """

    expires_at: Optional[datetime]
    """
    Expiration date of the service.
    """


@dataclass
class GetIPv6BlockQuotasResponseQuota:
    quota: int

    cidr: int


@dataclass
class InstallPartition:
    file_system: PartitionFileSystem
    """
    File system of the installation partition.
    """

    raid_level: RaidArrayRaidLevel
    """
    RAID level of the installation partition.
    """

    capacity: int
    """
    Capacity of the installation partition.
    """

    connectors: List[str]
    """
    Connectors of the installation partition.
    """

    mount_point: Optional[str]
    """
    Mount point of the installation partition.
    """


@dataclass
class FailoverIP:
    id: int
    """
    ID of the failover IP.
    """

    address: str
    """
    IP of the failover IP.
    """

    reverse: str
    """
    Reverse IP value.
    """

    ip_version: FailoverIPVersion
    """
    IP version of the failover IP.
    """

    cidr: int
    """
    Classless InterDomain Routing notation of the failover IP.
    """

    netmask: str
    """
    Netmask of the failover IP.
    """

    gateway_ip: str
    """
    Gateway IP of the failover IP.
    """

    status: FailoverIPStatus
    """
    Status of the IP failover.
    """

    type_: FailoverIPInterfaceType
    """
    The interface type.
    """

    mac: Optional[str]
    """
    MAC address of the IP failover.
    """

    server_id: Optional[int]
    """
    Server ID linked to the IP failover.
    """

    block: Optional[FailoverBlock]
    """
    Block of the IP failover.
    """

    server_zone: Optional[str]
    """
    The server zone (if assigned).
    """


@dataclass
class ListIPv6BlockSubnetsAvailableResponseSubnet:
    address: str

    cidr: int


@dataclass
class InvoiceSummary:
    id: int

    status: InvoiceStatus

    payment_method: InvoicePaymentMethod

    transaction_id: int

    total_with_taxes: Optional[Money]

    total_without_taxes: Optional[Money]

    created_at: Optional[datetime]

    paid_at: Optional[datetime]


@dataclass
class RpnSanIp:
    type_: RpnSanIpType
    """
    IP type (server | rpnv2_subnet).
    """

    ip: Optional[IP]
    """
    An IP object.
    """

    server: Optional[RpnSanIpServer]

    rpnv2_group: Optional[RpnSanIpRpnV2Group]


@dataclass
class RefundSummary:
    id: int

    status: RefundStatus

    method: RefundMethod

    total_with_taxes: Optional[Money]

    total_without_taxes: Optional[Money]

    created_at: Optional[datetime]

    refunded_at: Optional[datetime]


@dataclass
class RpnGroupMember:
    id: int
    """
    Rpn group member ID.
    """

    status: RpnGroupMemberStatus
    """
    RPN group member status.
    """

    group_id: int
    """
    RPN group ID.
    """

    group_name: str
    """
    RPN group name.
    """

    group_owner: str
    """
    RPN group owner.
    """

    owner: str
    """
    RPN member owner.
    """

    san_server: Optional[RpnSanServer]
    """
    Authorized RPN SAN server.
    """

    server: Optional[Server]
    """
    Authorized rpn v1 capable server.
    """

    speed: Optional[int]
    """
    RPN speed.
    """


@dataclass
class RpnSanSummary:
    id: int
    """
    RPN SAN  ID.
    """

    datacenter_name: str
    """
    Datacenter location.
    """

    organization_id: str
    """
    Organization ID.
    """

    project_id: str
    """
    Project ID.
    """

    server_hostname: str
    """
    RPN SAN server hostname.
    """

    iqn_suffix: str
    """
    IQN suffix.
    """

    offer_id: int
    """
    Offer ID.
    """

    created_at: Optional[datetime]
    """
    Date of creation of the RPN SAN.
    """

    offer_name: str
    """
    Offer description.
    """

    status: RpnSanStatus
    """
    Status.
    """

    storage_size: int
    """
    RPN SAN storage size.
    """

    rpnv1_compatible: bool
    """
    True if the SAN is compatible with the RPNv1 technology.
    """

    rpnv1_implicit: bool
    """
    True if the offer supports the RPNv1 implicitly, false if it must to be added to a group to support RPNv1.
    """

    delivered_at: Optional[datetime]
    """
    RPN SAN delivery date.
    """

    terminated_at: Optional[datetime]
    """
    RPN SAN termination date.
    """

    expires_at: Optional[datetime]
    """
    RPN SAN expiration date.
    """


@dataclass
class RpnServerCapability:
    id: int
    """
    Server ID.
    """

    hostname: str
    """
    Server hostname.
    """

    datacenter_name: str
    """
    Server datacenter name.
    """

    zone: ScwZone
    """
    Server zone.
    """

    compatible_qinq: bool
    """
    True if server is compatible with QinQ protocol (rpn v2).
    """

    can_join_qinq_group: bool
    """
    True if server can join a QinQ group.
    """

    rpnv1_group_count: int
    """
    Times server is linked in a rpnv1 group.
    """

    rpnv2_group_count: int
    """
    Times server is linked in a rpnv2 group.
    """

    can_join_rpnv2_group: bool
    """
    True if server can join an rpnv2 group.
    """

    ip_address: Optional[str]
    """
    Private IP address (if rpn compatiblle).
    """

    rpn_version: Optional[int]
    """
    Supported rpn version.
    """


@dataclass
class Log:
    id: int
    """
    RPN V2 log ID.
    """

    action: LogAction
    """
    Which action was performed.
    """

    status: LogStatus
    """
    Action status.
    """

    group: Optional[RpnV2Group]
    """
    RPN V2 group.
    """

    member: Optional[RpnV2Member]
    """
    RPN V2 member (if appliable).
    """

    created_at: Optional[datetime]
    """
    Creation date.
    """

    finished_at: Optional[datetime]
    """
    Completion date.
    """


@dataclass
class ServerEvent:
    event_id: int
    """
    ID of the event.
    """

    description: str
    """
    Description of the event.
    """

    date: Optional[datetime]
    """
    Date of the event.
    """


@dataclass
class ServerSummary:
    id: int
    """
    ID of the server.
    """

    datacenter_name: str
    """
    Datacenter of the server.
    """

    organization_id: str
    """
    Organization ID the server is attached to.
    """

    project_id: str
    """
    Project ID the server is attached to.
    """

    hostname: str
    """
    Hostname of the server.
    """

    created_at: Optional[datetime]
    """
    Date of creation of the server.
    """

    updated_at: Optional[datetime]
    """
    Date of last modification of the server.
    """

    expired_at: Optional[datetime]
    """
    Date of release of the server.
    """

    offer_id: int
    """
    Offer ID of the server.
    """

    offer_name: str
    """
    Offer name of the server.
    """

    status: ServerStatus
    """
    Status of the server.
    """

    interfaces: List[NetworkInterface]
    """
    Network interfaces of the server.
    """

    zone: ScwZone
    """
    The zone in which is the server.
    """

    is_outsourced: bool
    """
    Whether the server is outsourced or not.
    """

    qinq: bool
    """
    Whether the server is compatible with QinQ.
    """

    os_id: Optional[int]
    """
    OS ID installed on server.
    """

    level: Optional[ServiceLevel]
    """
    Service level of the server.
    """

    rpn_version: Optional[int]
    """
    Supported RPN version.
    """


@dataclass
class CPU:
    name: str
    """
    Name of CPU.
    """

    core_count: int
    """
    Number of cores of the CPU.
    """

    thread_count: int
    """
    Number of threads of the CPU.
    """

    frequency: int
    """
    Frequency of the CPU.
    """


@dataclass
class Disk:
    capacity: int
    """
    Capacity of the disk.
    """

    type_: ServerDiskType
    """
    Type of the disk.
    """


@dataclass
class Memory:
    capacity: int
    """
    Capacity of the memory.
    """

    type_: MemoryType
    """
    Type of the memory.
    """

    frequency: int
    """
    Frequency of the memory.
    """

    is_ecc: bool
    """
    True if the memory is an error-correcting code memory.
    """


@dataclass
class PersistentMemory:
    capacity: int
    """
    Capacity of the persistent memory.
    """

    frequency: int
    """
    Frequency of the persistent memory.
    """

    model: str
    """
    Model of the persistent memory.
    """


@dataclass
class RaidController:
    model: str
    """
    Model of the RAID controller.
    """

    raid_level: List[str]
    """
    RAID level of the RAID controller.
    """


@dataclass
class RaidArray:
    raid_level: RaidArrayRaidLevel
    """
    The RAID level.
    """

    disks: List[ServerDisk]
    """
    Disks on the RAID controller.
    """


@dataclass
class Partition:
    type_: PartitionType
    """
    Type of the partition.
    """

    file_system: PartitionFileSystem
    """
    File system of the partition.
    """

    raid_level: RaidArrayRaidLevel
    """
    Raid level of the partition.
    """

    capacity: int
    """
    Capacity of the partition.
    """

    connectors: List[str]
    """
    Connectors of the partition.
    """

    mount_point: Optional[str]
    """
    Mount point of the partition.
    """


@dataclass
class UpdatableRaidArray:
    raid_level: RaidArrayRaidLevel
    """
    The RAID level.
    """

    disk_ids: List[int]
    """
    The list of Disk ID of the updatable RAID.
    """


@dataclass
class AttachFailoverIPToMacAddressRequest:
    ip_id: int
    """
    ID of the failover IP.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    type_: Optional[AttachFailoverIPToMacAddressRequestMacType]
    """
    A mac type.
    """

    mac: Optional[str]
    """
    A valid mac address (existing or not).
    """


@dataclass
class AttachFailoverIPsRequest:
    server_id: int
    """
    ID of the server.
    """

    fips_ids: List[int]
    """
    List of ID of failovers IP to attach.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class BMCAccess:
    url: str
    """
    URL to access to the server console.
    """

    login: str
    """
    The login to use for the BMC (Baseboard Management Controller) access authentification.
    """

    password: str
    """
    The password to use for the BMC (Baseboard Management Controller) access authentification.
    """

    status: BMCAccessStatus
    """
    Status of the connection.
    """

    expires_at: Optional[datetime]
    """
    The date after which the BMC (Baseboard Management Controller) access will be closed.
    """


@dataclass
class Backup:
    id: int
    """
    ID of the backup.
    """

    login: str
    """
    Login of the backup.
    """

    server: str
    """
    Server of the backup.
    """

    status: BackupStatus
    """
    Status of the backup.
    """

    acl_enabled: bool
    """
    ACL enable boolean of the backup.
    """

    autologin: bool
    """
    Autologin boolean of the backup.
    """

    quota_space: int
    """
    Total quota space of the backup.
    """

    quota_space_used: int
    """
    Quota space used of the backup.
    """

    quota_files: int
    """
    Total quota files of the backup.
    """

    quota_files_used: int
    """
    Quota files used of the backup.
    """


@dataclass
class BillingApiCanOrderRequest:
    project_id: Optional[str]


@dataclass
class BillingApiDownloadInvoiceRequest:
    invoice_id: int


@dataclass
class BillingApiDownloadRefundRequest:
    refund_id: int


@dataclass
class BillingApiGetInvoiceRequest:
    invoice_id: int


@dataclass
class BillingApiGetRefundRequest:
    refund_id: int


@dataclass
class BillingApiListInvoicesRequest:
    page: Optional[int]

    page_size: Optional[int]

    order_by: Optional[ListInvoicesRequestOrderBy]

    project_id: Optional[str]


@dataclass
class BillingApiListRefundsRequest:
    page: Optional[int]

    page_size: Optional[int]

    order_by: Optional[ListRefundsRequestOrderBy]

    project_id: Optional[str]


@dataclass
class CanOrderResponse:
    can_order: bool

    quota_ok: bool

    phone_confirmed: bool

    email_confirmed: bool

    user_confirmed: bool

    payment_mode: bool

    billing_ok: bool

    message: Optional[str]


@dataclass
class CancelServerInstallRequest:
    server_id: int
    """
    Server ID of the server to cancel install.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class CreateFailoverIPsRequest:
    offer_id: int
    """
    Failover IP offer ID.
    """

    quantity: int
    """
    Quantity.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    project_id: Optional[str]
    """
    Project ID.
    """


@dataclass
class CreateFailoverIPsResponse:
    total_count: int

    services: List[Service]


@dataclass
class CreateServerRequest:
    offer_id: int
    """
    Offer ID of the new server.
    """

    server_option_ids: List[int]
    """
    Server option IDs of the new server.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    project_id: Optional[str]
    """
    Project ID of the new server.
    """

    datacenter_name: Optional[str]
    """
    Datacenter name of the new server.
    """


@dataclass
class DeleteFailoverIPRequest:
    ip_id: int
    """
    ID of the failover IP to delete.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class DeleteServerRequest:
    server_id: int
    """
    Server ID to delete.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class DeleteServiceRequest:
    service_id: int
    """
    ID of the service.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class DetachFailoverIPFromMacAddressRequest:
    ip_id: int
    """
    ID of the failover IP.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class DetachFailoverIPsRequest:
    fips_ids: List[int]
    """
    List of IDs of failovers IP to detach.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetBMCAccessRequest:
    server_id: int
    """
    ID of the server to get BMC access.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetFailoverIPRequest:
    ip_id: int
    """
    ID of the failover IP.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetIPv6BlockQuotasResponse:
    quotas: List[GetIPv6BlockQuotasResponseQuota]
    """
    Quota for each CIDR of IPv6 block.
    """

    total_count: int
    """
    Total count of quotas.
    """


@dataclass
class GetOSRequest:
    os_id: int
    """
    ID of the OS.
    """

    server_id: int
    """
    ID of the server.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    project_id: Optional[str]
    """
    Project ID.
    """


@dataclass
class GetOfferRequest:
    offer_id: int
    """
    ID of offer.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    project_id: Optional[str]
    """
    Project ID.
    """


@dataclass
class GetOrderedServiceRequest:
    ordered_service_id: int

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetRaidRequest:
    server_id: int
    """
    ID of the server.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetRemainingQuotaRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    project_id: Optional[str]
    """
    Project ID.
    """


@dataclass
class GetRemainingQuotaResponse:
    failover_ip_quota: int
    """
    Current failover IP quota.
    """

    failover_ip_remaining_quota: int
    """
    Remaining failover IP quota.
    """

    failover_block_quota: int
    """
    Current failover block quota.
    """

    failover_block_remaining_quota: int
    """
    Remaining failover block quota.
    """


@dataclass
class GetRescueRequest:
    server_id: int
    """
    ID of the server to get rescue.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetRpnStatusResponse:
    status: GetRpnStatusResponseStatus
    """
    If status = 'operational', you can perform rpn actions in write.
    """

    operations_left: Optional[int]
    """
    Number of operations left to perform before being operational.
    """


@dataclass
class GetServerBackupRequest:
    server_id: int
    """
    Server ID of the backup.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetServerDefaultPartitioningRequest:
    server_id: int
    """
    ID of the server.
    """

    os_id: int
    """
    OS ID of the default partitioning.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetServerInstallRequest:
    server_id: int
    """
    Server ID of the server to install.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetServerRequest:
    server_id: int
    """
    ID of the server.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetServiceRequest:
    service_id: int
    """
    ID of the service.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class IPv6Block:
    id: int
    """
    ID of the IPv6.
    """

    address: str
    """
    Address of the IPv6.
    """

    duid: str
    """
    DUID of the IPv6.
    """

    nameservers: List[str]
    """
    DNS linked to the IPv6.
    """

    cidr: int
    """
    Classless InterDomain Routing notation of the IPv6.
    """

    subnets: List[IPv6Block]
    """
    All IPv6 subnets.
    """

    delegation_status: IPv6BlockDelegationStatus
    """
    The nameservers delegation status.
    """


@dataclass
class IPv6BlockApiCreateIPv6BlockRequest:
    project_id: Optional[str]
    """
    ID of the project.
    """


@dataclass
class IPv6BlockApiCreateIPv6BlockSubnetRequest:
    block_id: int
    """
    ID of the IPv6 block.
    """

    address: str
    """
    Address of the IPv6.
    """

    cidr: int
    """
    Classless InterDomain Routing notation of the IPv6.
    """


@dataclass
class IPv6BlockApiDeleteIPv6BlockRequest:
    block_id: int
    """
    ID of the IPv6 block to delete.
    """


@dataclass
class IPv6BlockApiGetIPv6BlockQuotasRequest:
    project_id: Optional[str]
    """
    ID of the project.
    """


@dataclass
class IPv6BlockApiGetIPv6BlockRequest:
    project_id: Optional[str]
    """
    ID of the project.
    """


@dataclass
class IPv6BlockApiListIPv6BlockSubnetsAvailableRequest:
    block_id: int
    """
    ID of the IPv6 block.
    """


@dataclass
class IPv6BlockApiUpdateIPv6BlockRequest:
    block_id: int
    """
    ID of the IPv6 block.
    """

    nameservers: Optional[List[str]]
    """
    DNS to link to the IPv6.
    """


@dataclass
class InstallServerRequest:
    server_id: int
    """
    Server ID to install.
    """

    os_id: int
    """
    OS ID to install on the server.
    """

    hostname: str
    """
    Hostname of the server.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    user_login: Optional[str]
    """
    User to install on the server.
    """

    user_password: Optional[str]
    """
    User password to install on the server.
    """

    panel_password: Optional[str]
    """
    Panel password to install on the server.
    """

    root_password: Optional[str]
    """
    Root password to install on the server.
    """

    partitions: Optional[List[InstallPartition]]
    """
    Partitions to install on the server.
    """

    ssh_key_ids: Optional[List[str]]
    """
    SSH key IDs authorized on the server.
    """

    license_offer_id: Optional[int]
    """
    Offer ID of license to install on server.
    """

    ip_id: Optional[int]
    """
    IP to link at the license to install on server.
    """


@dataclass
class Invoice:
    id: int

    status: InvoiceStatus

    payment_method: InvoicePaymentMethod

    content: str

    transaction_id: int

    total_with_taxes: Optional[Money]

    total_without_taxes: Optional[Money]

    created_at: Optional[datetime]

    paid_at: Optional[datetime]


@dataclass
class ListFailoverIPsRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of failovers IP per page.
    """

    order_by: Optional[ListFailoverIPsRequestOrderBy]
    """
    Order of the failovers IP.
    """

    project_id: Optional[str]
    """
    Filter failovers IP by project ID.
    """

    search: Optional[str]
    """
    Filter failovers IP which matching with this field.
    """

    only_available: Optional[bool]
    """
    True: return all failovers IP not attached on server
false: return all failovers IP attached on server.
    """


@dataclass
class ListFailoverIPsResponse:
    total_count: int
    """
    Total count of matching failovers IP.
    """

    failover_ips: List[FailoverIP]
    """
    List of failover IPs that match filters.
    """


@dataclass
class ListIPv6BlockSubnetsAvailableResponse:
    subnet_availables: List[ListIPv6BlockSubnetsAvailableResponseSubnet]
    """
    All available address and CIDR available in subnet.
    """

    total_count: int
    """
    Total count of available subnets.
    """


@dataclass
class ListInvoicesResponse:
    total_count: int

    invoices: List[InvoiceSummary]


@dataclass
class ListIpsResponse:
    total_count: int
    """
    Total count of authorized IPs.
    """

    ips: List[RpnSanIp]
    """
    List of authorized IPs.
    """


@dataclass
class ListOSRequest:
    server_id: int
    """
    Filter OS by compatible server ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of OS per page.
    """

    order_by: Optional[ListOSRequestOrderBy]
    """
    Order of the OS.
    """

    type_: Optional[OSType]
    """
    Type of the OS.
    """

    project_id: Optional[str]
    """
    Project ID.
    """


@dataclass
class ListOSResponse:
    total_count: int
    """
    Total count of matching OS.
    """

    os: List[OS]
    """
    OS that match filters.
    """


@dataclass
class ListOffersRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of offer per page.
    """

    order_by: Optional[ListOffersRequestOrderBy]
    """
    Order of the offers.
    """

    commercial_range: Optional[str]
    """
    Filter on commercial range.
    """

    catalog: Optional[OfferCatalog]
    """
    Filter on catalog.
    """

    project_id: Optional[str]
    """
    Project ID.
    """

    is_failover_ip: Optional[bool]
    """
    Get the current failover IP offer.
    """

    is_failover_block: Optional[bool]
    """
    Get the current failover IP block offer.
    """

    sold_in: Optional[List[str]]
    """
    Filter offers depending on their datacenter.
    """

    available_only: Optional[bool]
    """
    Set this filter to true to only return available offers.
    """

    is_rpn_san: Optional[bool]
    """
    Get the RPN SAN offers.
    """


@dataclass
class ListOffersResponse:
    total_count: int
    """
    Total count of matching offers.
    """

    offers: List[Offer]
    """
    Offers that match filters.
    """


@dataclass
class ListRefundsResponse:
    total_count: int

    refunds: List[RefundSummary]


@dataclass
class ListRpnCapableSanServersResponse:
    total_count: int
    """
    Total count of rpn capable san servers.
    """

    san_servers: List[RpnSanServer]
    """
    List of san servers.
    """


@dataclass
class ListRpnCapableServersResponse:
    total_count: int
    """
    Total count of rpn capable servers.
    """

    servers: List[Server]
    """
    List of servers.
    """


@dataclass
class ListRpnGroupMembersResponse:
    total_count: int
    """
    Total count of rpn v1 group members.
    """

    members: List[RpnGroupMember]
    """
    List of rpn v1 group members.
    """


@dataclass
class ListRpnGroupsResponse:
    total_count: int
    """
    Total count of rpn groups.
    """

    rpn_groups: List[RpnGroup]
    """
    List of rpn v1 groups.
    """


@dataclass
class ListRpnInvitesResponse:
    total_count: int
    """
    Total count of invites.
    """

    members: List[RpnGroupMember]
    """
    List of invites.
    """


@dataclass
class ListRpnSansResponse:
    total_count: int
    """
    Total count of matching RPN SANs.
    """

    rpn_sans: List[RpnSanSummary]
    """
    List of RPN SANs that match filters.
    """


@dataclass
class ListRpnServerCapabilitiesResponse:
    total_count: int
    """
    Total count of servers.
    """

    servers: List[RpnServerCapability]
    """
    List of servers and their RPN capabilities.
    """


@dataclass
class ListRpnV2CapableResourcesResponse:
    total_count: int
    """
    Total count of matching rpn v2 capable resources.
    """

    servers: List[Server]
    """
    List of rpn v2 capable resources that match filters.
    """


@dataclass
class ListRpnV2GroupLogsResponse:
    total_count: int
    """
    Total count of matching rpn v2 logs.
    """

    logs: List[Log]
    """
    List of rpn v2 logs that match filters.
    """


@dataclass
class ListRpnV2GroupsResponse:
    total_count: int
    """
    Total count of matching rpn v2 groups.
    """

    rpn_groups: List[RpnV2Group]
    """
    List of rpn v2 groups that match filters.
    """


@dataclass
class ListRpnV2MembersResponse:
    total_count: int
    """
    Total count of matching rpn v2 group members.
    """

    members: List[RpnV2Member]
    """
    List of rpn v2 group members that match filters.
    """


@dataclass
class ListServerDisksRequest:
    server_id: int
    """
    Server ID of the server disks.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of server disk per page.
    """

    order_by: Optional[ListServerDisksRequestOrderBy]
    """
    Order of the server disks.
    """


@dataclass
class ListServerDisksResponse:
    total_count: int
    """
    Total count of matching server disks.
    """

    disks: List[ServerDisk]
    """
    Server disks that match filters.
    """


@dataclass
class ListServerEventsRequest:
    server_id: int
    """
    Server ID of the server events.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of server event per page.
    """

    order_by: Optional[ListServerEventsRequestOrderBy]
    """
    Order of the server events.
    """


@dataclass
class ListServerEventsResponse:
    total_count: int
    """
    Total count of matching server events.
    """

    events: List[ServerEvent]
    """
    Server events that match filters.
    """


@dataclass
class ListServersRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of server per page.
    """

    order_by: Optional[ListServersRequestOrderBy]
    """
    Order of the servers.
    """

    project_id: Optional[str]
    """
    Filter servers by project ID.
    """

    search: Optional[str]
    """
    Filter servers by hostname.
    """


@dataclass
class ListServersResponse:
    total_count: int
    """
    Total count of matching servers.
    """

    servers: List[ServerSummary]
    """
    Servers that match filters.
    """


@dataclass
class ListServicesRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of service per page.
    """

    order_by: Optional[ListServicesRequestOrderBy]
    """
    Order of the services.
    """

    project_id: Optional[str]
    """
    Project ID.
    """


@dataclass
class ListServicesResponse:
    total_count: int
    """
    Total count of matching services.
    """

    services: List[Service]
    """
    Services that match filters.
    """


@dataclass
class ListSubscribableServerOptionsRequest:
    server_id: int
    """
    Server ID of the subscribable server options.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of subscribable server option per page.
    """


@dataclass
class ListSubscribableServerOptionsResponse:
    total_count: int
    """
    Total count of matching subscribable server options.
    """

    server_options: List[Offer]
    """
    Server options that match filters.
    """


@dataclass
class OfferFailoverBlockInfo:
    onetime_fees: Optional[Offer]


@dataclass
class OfferFailoverIpInfo:
    onetime_fees: Optional[Offer]


@dataclass
class OfferServerInfo:
    bandwidth: int

    stock: OfferServerInfoStock

    commercial_range: str

    disks: List[Disk]

    cpus: List[CPU]

    memories: List[Memory]

    persistent_memories: List[PersistentMemory]

    raid_controllers: List[RaidController]

    available_options: List[Offer]

    connectivity: int

    stock_by_datacenter: Dict[str, OfferServerInfoStock]

    rpn_version: Optional[int]

    onetime_fees: Optional[Offer]


@dataclass
class OfferServiceLevelInfo:
    support_ticket: bool

    support_phone: bool

    sales_support: bool

    git: str

    sla: float

    priority_support: bool

    high_rpn_bandwidth: bool

    customization: bool

    antidos: bool

    extra_failover_quota: int

    available_options: List[Offer]


@dataclass
class Raid:
    raid_arrays: List[RaidArray]
    """
    Details about the RAID controller.
    """


@dataclass
class RebootServerRequest:
    server_id: int
    """
    Server ID to reboot.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class Refund:
    id: int

    status: RefundStatus

    method: RefundMethod

    content: str

    total_with_taxes: Optional[Money]

    total_without_taxes: Optional[Money]

    created_at: Optional[datetime]

    refunded_at: Optional[datetime]


@dataclass
class Rescue:
    os_id: int
    """
    OS ID of the rescue.
    """

    login: str
    """
    Login of the rescue.
    """

    password: str
    """
    Password of the rescue.
    """

    protocol: RescueProtocol
    """
    Protocol of the resuce.
    """


@dataclass
class RpnApiGetRpnStatusRequest:
    project_id: Optional[str]
    """
    A project ID.
    """

    rpnv1_group_id: Optional[int]
    """
    An RPN v1 group ID.
    """

    rpnv2_group_id: Optional[int]
    """
    An RPN v2 group ID.
    """


@dataclass
class RpnApiListRpnServerCapabilitiesRequest:
    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of servers per page.
    """

    order_by: Optional[ListRpnServerCapabilitiesRequestOrderBy]
    """
    Order of the servers.
    """

    project_id: Optional[str]
    """
    Filter servers by project ID.
    """


@dataclass
class RpnSanApiAddIpRequest:
    rpn_san_id: int
    """
    RPN SAN ID.
    """

    ip_ids: List[int]
    """
    An array of IP ID.
    """


@dataclass
class RpnSanApiCreateRpnSanRequest:
    offer_id: int
    """
    Offer ID.
    """

    project_id: Optional[str]
    """
    Your project ID.
    """


@dataclass
class RpnSanApiDeleteRpnSanRequest:
    rpn_san_id: int
    """
    RPN SAN ID.
    """


@dataclass
class RpnSanApiGetRpnSanRequest:
    rpn_san_id: int
    """
    RPN SAN ID.
    """


@dataclass
class RpnSanApiListAvailableIpsRequest:
    rpn_san_id: int
    """
    RPN SAN ID.
    """

    type_: Optional[RpnSanIpType]
    """
    Filter by IP type (server | rpnv2_subnet).
    """


@dataclass
class RpnSanApiListIpsRequest:
    rpn_san_id: int
    """
    RPN SAN ID.
    """

    type_: Optional[RpnSanIpType]
    """
    Filter by IP type (server | rpnv2_subnet).
    """


@dataclass
class RpnSanApiListRpnSansRequest:
    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of RPN SANs per page.
    """

    order_by: Optional[ListRpnSansRequestOrderBy]
    """
    Order of the RPN SANs.
    """

    project_id: Optional[str]
    """
    Filter RPN SANs by project ID.
    """


@dataclass
class RpnSanApiRemoveIpRequest:
    rpn_san_id: int
    """
    RPN SAN ID.
    """

    ip_ids: List[int]
    """
    An array of IP ID.
    """


@dataclass
class RpnV1ApiAcceptRpnInviteRequest:
    member_id: int
    """
    The member ID.
    """


@dataclass
class RpnV1ApiAddRpnGroupMembersRequest:
    group_id: int
    """
    The rpn v1 group ID.
    """

    server_ids: Optional[List[int]]
    """
    A collection of rpn v1 capable server IDs.
    """

    san_server_ids: Optional[List[int]]
    """
    A collection of rpn v1 capable RPN SAN server IDs.
    """


@dataclass
class RpnV1ApiCreateRpnGroupRequest:
    name: str
    """
    Rpn v1 group name.
    """

    server_ids: Optional[List[int]]
    """
    A collection of rpn v1 capable servers.
    """

    san_server_ids: Optional[List[int]]
    """
    A collection of rpn v1 capable rpn sans servers.
    """

    project_id: Optional[str]
    """
    A project ID.
    """


@dataclass
class RpnV1ApiDeleteRpnGroupMembersRequest:
    group_id: int
    """
    The rpn v1 group ID.
    """

    member_ids: List[int]
    """
    A collection of rpn v1 group members IDs.
    """


@dataclass
class RpnV1ApiDeleteRpnGroupRequest:
    group_id: int
    """
    Rpn v1 group ID.
    """


@dataclass
class RpnV1ApiGetRpnGroupRequest:
    group_id: int
    """
    Rpn v1 group ID.
    """


@dataclass
class RpnV1ApiLeaveRpnGroupRequest:
    group_id: int
    """
    The RPN V1 group ID.
    """

    member_ids: List[int]
    """
    A collection of rpn v1 group members IDs.
    """

    project_id: Optional[str]
    """
    A project ID.
    """


@dataclass
class RpnV1ApiListRpnCapableSanServersRequest:
    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of rpn capable resources per page.
    """

    order_by: Optional[ListRpnCapableSanServersRequestOrderBy]
    """
    Order of the rpn capable resources.
    """

    project_id: Optional[str]
    """
    Filter rpn capable resources by project ID.
    """


@dataclass
class RpnV1ApiListRpnCapableServersRequest:
    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of rpn capable resources per page.
    """

    order_by: Optional[ListRpnCapableServersRequestOrderBy]
    """
    Order of the rpn capable resources.
    """

    project_id: Optional[str]
    """
    Filter rpn capable resources by project ID.
    """


@dataclass
class RpnV1ApiListRpnGroupMembersRequest:
    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of rpn v1 group members per page.
    """

    order_by: Optional[ListRpnGroupMembersRequestOrderBy]
    """
    Order of the rpn v1 group members.
    """

    group_id: int
    """
    Filter rpn v1 group members by group ID.
    """

    project_id: Optional[str]
    """
    A project ID.
    """


@dataclass
class RpnV1ApiListRpnGroupsRequest:
    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of rpn v1 groups per page.
    """

    order_by: Optional[ListRpnGroupsRequestOrderBy]
    """
    Order of the rpn v1 groups.
    """

    project_id: Optional[str]
    """
    Filter rpn v1 groups by project ID.
    """


@dataclass
class RpnV1ApiListRpnInvitesRequest:
    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of rpn capable resources per page.
    """

    order_by: Optional[ListRpnInvitesRequestOrderBy]
    """
    Order of the rpn capable resources.
    """

    project_id: Optional[str]
    """
    Filter rpn capable resources by project ID.
    """


@dataclass
class RpnV1ApiRefuseRpnInviteRequest:
    member_id: int
    """
    The member ID.
    """


@dataclass
class RpnV1ApiRpnGroupInviteRequest:
    group_id: int
    """
    The RPN V1 group ID.
    """

    server_ids: List[int]
    """
    A collection of external server IDs.
    """

    project_id: Optional[str]
    """
    A project ID.
    """


@dataclass
class RpnV1ApiUpdateRpnGroupNameRequest:
    group_id: int
    """
    Rpn v1 group ID.
    """

    name: Optional[str]
    """
    New rpn v1 group name.
    """


@dataclass
class RpnV2ApiAddRpnV2MembersRequest:
    group_id: int
    """
    RPN V2 group ID.
    """

    servers: List[int]
    """
    A collection of server IDs.
    """


@dataclass
class RpnV2ApiCreateRpnV2GroupRequest:
    name: str
    """
    RPN V2 group name.
    """

    servers: List[int]
    """
    A collection of server IDs.
    """

    project_id: Optional[str]
    """
    Project ID of the RPN V2 group.
    """

    type_: Optional[RpnV2GroupType]
    """
    RPN V2 group type (qing / standard).
    """


@dataclass
class RpnV2ApiDeleteRpnV2GroupRequest:
    group_id: int
    """
    RPN V2 group ID.
    """


@dataclass
class RpnV2ApiDeleteRpnV2MembersRequest:
    group_id: int
    """
    RPN V2 group ID.
    """

    member_ids: List[int]
    """
    A collection of member IDs.
    """


@dataclass
class RpnV2ApiDisableRpnV2GroupCompatibilityRequest:
    group_id: int
    """
    RPN V2 group ID.
    """


@dataclass
class RpnV2ApiEnableRpnV2GroupCompatibilityRequest:
    group_id: int
    """
    RPN V2 group ID.
    """

    rpnv1_group_id: int
    """
    RPN V1 group ID.
    """


@dataclass
class RpnV2ApiGetRpnV2GroupRequest:
    group_id: int
    """
    RPN V2 group ID.
    """


@dataclass
class RpnV2ApiListRpnV2CapableResourcesRequest:
    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of rpn v2 capable resources per page.
    """

    order_by: Optional[ListRpnV2CapableResourcesRequestOrderBy]
    """
    Order of the rpn v2 capable resources.
    """

    project_id: Optional[str]
    """
    Filter rpn v2 capable resources by project ID.
    """


@dataclass
class RpnV2ApiListRpnV2GroupLogsRequest:
    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of rpn v2 group logs per page.
    """

    order_by: Optional[ListRpnV2GroupLogsRequestOrderBy]
    """
    Order of the rpn v2 group logs.
    """

    group_id: int
    """
    RPN V2 group ID.
    """


@dataclass
class RpnV2ApiListRpnV2GroupsRequest:
    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of rpn v2 groups per page.
    """

    order_by: Optional[ListRpnV2GroupsRequestOrderBy]
    """
    Order of the rpn v2 groups.
    """

    project_id: Optional[str]
    """
    Filter rpn v2 groups by project ID.
    """


@dataclass
class RpnV2ApiListRpnV2MembersRequest:
    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Number of rpn v2 group members per page.
    """

    order_by: Optional[ListRpnV2MembersRequestOrderBy]
    """
    Order of the rpn v2 group members.
    """

    group_id: int
    """
    RPN V2 group ID.
    """

    type_: Optional[ListRpnV2MembersRequestType]
    """
    Filter members by type.
    """


@dataclass
class RpnV2ApiUpdateRpnV2GroupNameRequest:
    group_id: int
    """
    RPN V2 group ID.
    """

    name: Optional[str]
    """
    RPN V2 group name.
    """


@dataclass
class RpnV2ApiUpdateRpnV2VlanForMembersRequest:
    group_id: int
    """
    RPN V2 group ID.
    """

    member_ids: List[int]
    """
    RPN V2 member IDs.
    """

    vlan: Optional[int]
    """
    Min: 0.
Max: 3967.
    """


@dataclass
class ServerDefaultPartitioning:
    partitions: List[Partition]
    """
    Default partitions.
    """


@dataclass
class ServerInstall:
    os_id: int

    hostname: str

    partitions: List[Partition]

    ssh_key_ids: List[str]

    status: ServerInstallStatus

    user_login: Optional[str]

    panel_url: Optional[str]


@dataclass
class StartBMCAccessRequest:
    server_id: int
    """
    ID of the server to start the BMC access.
    """

    ip: str
    """
    The IP authorized to connect to the given server.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class StartRescueRequest:
    server_id: int
    """
    ID of the server to start rescue.
    """

    os_id: int
    """
    OS ID to use to start rescue.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class StartServerRequest:
    server_id: int
    """
    Server ID to start.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class StopBMCAccessRequest:
    server_id: int
    """
    ID of the server to stop BMC access.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class StopRescueRequest:
    server_id: int
    """
    ID of the server to stop rescue.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class StopServerRequest:
    server_id: int
    """
    Server ID to stop.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class SubscribeServerOptionRequest:
    server_id: int
    """
    Server ID to subscribe server option.
    """

    option_id: int
    """
    Option ID to subscribe.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class SubscribeStorageOptionsRequest:
    server_id: int
    """
    Server ID of the storage options to subscribe.
    """

    options_ids: List[int]
    """
    Option IDs of the storage options to subscribe.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class SubscribeStorageOptionsResponse:
    services: List[Service]
    """
    Services subscribe storage options.
    """


@dataclass
class UpdateRaidRequest:
    server_id: int
    """
    ID of the server.
    """

    raid_arrays: List[UpdatableRaidArray]
    """
    RAIDs to update.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class UpdateReverseRequest:
    ip_id: int
    """
    ID of the IP.
    """

    reverse: str
    """
    Reverse to apply on the IP.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class UpdateServerBackupRequest:
    server_id: int
    """
    Server ID to update backup.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    password: Optional[str]
    """
    Password of the server backup.
    """

    autologin: Optional[bool]
    """
    Autologin of the server backup.
    """

    acl_enabled: Optional[bool]
    """
    Boolean to enable or disable ACL.
    """


@dataclass
class UpdateServerRequest:
    server_id: int
    """
    Server ID to update.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    hostname: Optional[str]
    """
    Hostname of the server to update.
    """

    enable_ipv6: Optional[bool]
    """
    Flag to enable or not the IPv6 of server.
    """


@dataclass
class UpdateServerTagsRequest:
    server_id: int
    """
    Server ID to update the tags.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    tags: Optional[List[str]]
    """
    Tags of server to update.
    """



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import List, Optional

from scaleway_core.api import API
from scaleway_core.bridge import (
    ScwFile,
    unmarshal_ScwFile,
)
from scaleway_core.utils import (
    random_name,
    validate_path_param,
    fetch_all_pages,
)
from .types import (
    ContractType,
    ListContractSignaturesRequestOrderBy,
    ListProjectsRequestOrderBy,
    CheckContractSignatureResponse,
    ContractApiCheckContractSignatureRequest,
    ContractApiCreateContractSignatureRequest,
    ContractSignature,
    ListContractSignaturesResponse,
    ListProjectsResponse,
    Project,
    ProjectApiCreateProjectRequest,
    ProjectApiSetProjectQualificationRequest,
    ProjectApiUpdateProjectRequest,
    ProjectQualification,
    Qualification,
)
from .marshalling import (
    unmarshal_ContractSignature,
    unmarshal_Project,
    unmarshal_CheckContractSignatureResponse,
    unmarshal_ListContractSignaturesResponse,
    unmarshal_ListProjectsResponse,
    unmarshal_ProjectQualification,
    marshal_ContractApiCheckContractSignatureRequest,
    marshal_ContractApiCreateContractSignatureRequest,
    marshal_ProjectApiCreateProjectRequest,
    marshal_ProjectApiSetProjectQualificationRequest,
    marshal_ProjectApiUpdateProjectRequest,
)
from ...std.types import (
    LanguageCode as StdLanguageCode,
)


class AccountV3ContractAPI(API):
    """
    The Contract API allows you to manage contracts.
    """

    def download_contract_signature(
        self,
        *,
        contract_signature_id: str,
        locale: Optional[StdLanguageCode] = None,
    ) -> ScwFile:
        """
        Download a contract content.
        :param contract_signature_id: The contract signature ID.
        :param locale: The locale requested for the content of the contract.
        :return: :class:`ScwFile <ScwFile>`

        Usage:
        ::

            result = api.download_contract_signature(
                contract_signature_id="example",
            )
        """

        param_contract_signature_id = validate_path_param(
            "contract_signature_id", contract_signature_id
        )

        res = self._request(
            "GET",
            f"/account/v3/contract-signatures/{param_contract_signature_id}/download",
            params={
                "locale": locale,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ScwFile(res.json())

    def create_contract_signature(
        self,
        *,
        contract_name: str,
        validated: bool,
        contract_type: Optional[ContractType] = None,
        organization_id: Optional[str] = None,
    ) -> ContractSignature:
        """
        Create a signature for your Organization for the latest version of the requested contract.
        :param contract_name: The name of the contract.
        :param validated: Whether the contract is validated at creation.
        :param contract_type: The type of the contract.
        :param organization_id: ID of the Organization.
        :return: :class:`ContractSignature <ContractSignature>`

        Usage:
        ::

            result = api.create_contract_signature(
                contract_name="example",
                validated=False,
            )
        """

        res = self._request(
            "POST",
            "/account/v3/contract-signatures",
            body=marshal_ContractApiCreateContractSignatureRequest(
                ContractApiCreateContractSignatureRequest(
                    contract_name=contract_name,
                    validated=validated,
                    contract_type=contract_type,
                    organization_id=organization_id,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_ContractSignature(res.json())

    def validate_contract_signature(
        self,
        *,
        contract_signature_id: str,
    ) -> ContractSignature:
        """
        Sign a contract for your Organization.
        :param contract_signature_id: The contract linked to your Organization you want to sign.
        :return: :class:`ContractSignature <ContractSignature>`

        Usage:
        ::

            result = api.validate_contract_signature(
                contract_signature_id="example",
            )
        """

        param_contract_signature_id = validate_path_param(
            "contract_signature_id", contract_signature_id
        )

        res = self._request(
            "POST",
            f"/account/v3/contract-signatures/{param_contract_signature_id}/validate",
            body={},
        )

        self._throw_on_error(res)
        return unmarshal_ContractSignature(res.json())

    def check_contract_signature(
        self,
        *,
        contract_name: str,
        organization_id: Optional[str] = None,
        contract_type: Optional[ContractType] = None,
    ) -> CheckContractSignatureResponse:
        """
        Check if a contract is signed for your Organization.
        :param contract_name: Filter on contract name.
        :param organization_id: ID of the Organization to check the contract signature for.
        :param contract_type: Filter on contract type.
        :return: :class:`CheckContractSignatureResponse <CheckContractSignatureResponse>`

        Usage:
        ::

            result = api.check_contract_signature(
                contract_name="example",
            )
        """

        res = self._request(
            "POST",
            "/account/v3/contract-signatures/check",
            body=marshal_ContractApiCheckContractSignatureRequest(
                ContractApiCheckContractSignatureRequest(
                    contract_name=contract_name,
                    organization_id=organization_id,
                    contract_type=contract_type,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_CheckContractSignatureResponse(res.json())

    def list_contract_signatures(
        self,
        *,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListContractSignaturesRequestOrderBy] = None,
        organization_id: Optional[str] = None,
    ) -> ListContractSignaturesResponse:
        """
        List contract signatures for an Organization.
        :param page: The page number for the returned contracts.
        :param page_size: The maximum number of contracts per page.
        :param order_by: How the contracts are ordered in the response.
        :param organization_id: Filter on Organization ID.
        :return: :class:`ListContractSignaturesResponse <ListContractSignaturesResponse>`

        Usage:
        ::

            result = api.list_contract_signatures()
        """

        res = self._request(
            "GET",
            "/account/v3/contract-signatures",
            params={
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListContractSignaturesResponse(res.json())

    def list_contract_signatures_all(
        self,
        *,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListContractSignaturesRequestOrderBy] = None,
        organization_id: Optional[str] = None,
    ) -> List[ContractSignature]:
        """
        List contract signatures for an Organization.
        :param page: The page number for the returned contracts.
        :param page_size: The maximum number of contracts per page.
        :param order_by: How the contracts are ordered in the response.
        :param organization_id: Filter on Organization ID.
        :return: :class:`List[ContractSignature] <List[ContractSignature]>`

        Usage:
        ::

            result = api.list_contract_signatures_all()
        """

        return fetch_all_pages(
            type=ListContractSignaturesResponse,
            key="contract_signatures",
            fetcher=self.list_contract_signatures,
            args={
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
                "organization_id": organization_id,
            },
        )


class AccountV3ProjectAPI(API):
    """
    This API allows you to manage your Scaleway Projects.
    """

    def create_project(
        self,
        *,
        description: str,
        name: Optional[str] = None,
        organization_id: Optional[str] = None,
    ) -> Project:
        """
        Create a new Project for an Organization.
        Generate a new Project for an Organization, specifying its configuration including name and description.
        :param description: Description of the Project.
        :param name: Name of the Project.
        :param organization_id: Organization ID of the Project.
        :return: :class:`Project <Project>`

        Usage:
        ::

            result = api.create_project(
                description="example",
            )
        """

        res = self._request(
            "POST",
            "/account/v3/projects",
            body=marshal_ProjectApiCreateProjectRequest(
                ProjectApiCreateProjectRequest(
                    description=description,
                    name=name or random_name(prefix="proj"),
                    organization_id=organization_id,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Project(res.json())

    def list_projects(
        self,
        *,
        organization_id: Optional[str] = None,
        name: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListProjectsRequestOrderBy] = None,
        project_ids: Optional[List[str]] = None,
    ) -> ListProjectsResponse:
        """
        List all Projects of an Organization.
        List all Projects of an Organization. The response will include the total number of Projects as well as their associated Organizations, names, and IDs. Other information includes the creation and update date of the Project.
        :param organization_id: Organization ID of the Project.
        :param name: Name of the Project.
        :param page: Page number for the returned Projects.
        :param page_size: Maximum number of Project per page.
        :param order_by: Sort order of the returned Projects.
        :param project_ids: Project IDs to filter for. The results will be limited to any Projects with an ID in this array.
        :return: :class:`ListProjectsResponse <ListProjectsResponse>`

        Usage:
        ::

            result = api.list_projects()
        """

        res = self._request(
            "GET",
            "/account/v3/projects",
            params={
                "name": name,
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "project_ids": project_ids,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListProjectsResponse(res.json())

    def list_projects_all(
        self,
        *,
        organization_id: Optional[str] = None,
        name: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListProjectsRequestOrderBy] = None,
        project_ids: Optional[List[str]] = None,
    ) -> List[Project]:
        """
        List all Projects of an Organization.
        List all Projects of an Organization. The response will include the total number of Projects as well as their associated Organizations, names, and IDs. Other information includes the creation and update date of the Project.
        :param organization_id: Organization ID of the Project.
        :param name: Name of the Project.
        :param page: Page number for the returned Projects.
        :param page_size: Maximum number of Project per page.
        :param order_by: Sort order of the returned Projects.
        :param project_ids: Project IDs to filter for. The results will be limited to any Projects with an ID in this array.
        :return: :class:`List[Project] <List[Project]>`

        Usage:
        ::

            result = api.list_projects_all()
        """

        return fetch_all_pages(
            type=ListProjectsResponse,
            key="projects",
            fetcher=self.list_projects,
            args={
                "organization_id": organization_id,
                "name": name,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
                "project_ids": project_ids,
            },
        )

    def get_project(
        self,
        *,
        project_id: Optional[str] = None,
    ) -> Project:
        """
        Get an existing Project.
        Retrieve information about an existing Project, specified by its Project ID. Its full details, including ID, name and description, are returned in the response object.
        :param project_id: Project ID of the Project.
        :return: :class:`Project <Project>`

        Usage:
        ::

            result = api.get_project()
        """

        param_project_id = validate_path_param(
            "project_id", project_id or self.client.default_project_id
        )

        res = self._request(
            "GET",
            f"/account/v3/projects/{param_project_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Project(res.json())

    def delete_project(
        self,
        *,
        project_id: Optional[str] = None,
    ) -> None:
        """
        Delete an existing Project.
        Delete an existing Project, specified by its Project ID. The Project needs to be empty (meaning there are no resources left in it) to be deleted effectively. Note that deleting a Project is permanent, and cannot be undone.
        :param project_id: Project ID of the Project.

        Usage:
        ::

            result = api.delete_project()
        """

        param_project_id = validate_path_param(
            "project_id", project_id or self.client.default_project_id
        )

        res = self._request(
            "DELETE",
            f"/account/v3/projects/{param_project_id}",
        )

        self._throw_on_error(res)

    def update_project(
        self,
        *,
        project_id: Optional[str] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
    ) -> Project:
        """
        Update Project.
        Update the parameters of an existing Project, specified by its Project ID. These parameters include the name and description.
        :param project_id: Project ID of the Project.
        :param name: Name of the Project.
        :param description: Description of the Project.
        :return: :class:`Project <Project>`

        Usage:
        ::

            result = api.update_project()
        """

        param_project_id = validate_path_param(
            "project_id", project_id or self.client.default_project_id
        )

        res = self._request(
            "PATCH",
            f"/account/v3/projects/{param_project_id}",
            body=marshal_ProjectApiUpdateProjectRequest(
                ProjectApiUpdateProjectRequest(
                    project_id=project_id,
                    name=name,
                    description=description,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Project(res.json())

    def set_project_qualification(
        self,
        *,
        project_id: Optional[str] = None,
        qualification: Optional[Qualification] = None,
    ) -> ProjectQualification:
        """
        Set project use case.
        Set the project use case for a new or existing Project, specified by its Project ID. You can customize the use case, sub use case, and architecture type you want to use in the Project.
        :param project_id: Project ID.
        :param qualification: Use case chosen for the Project.
        :return: :class:`ProjectQualification <ProjectQualification>`

        Usage:
        ::

            result = api.set_project_qualification()
        """

        param_project_id = validate_path_param(
            "project_id", project_id or self.client.default_project_id
        )

        res = self._request(
            "POST",
            f"/account/v3/projects/{param_project_id}/project-qualification",
            body=marshal_ProjectApiSetProjectQualificationRequest(
                ProjectApiSetProjectQualificationRequest(
                    project_id=project_id,
                    qualification=qualification,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_ProjectQualification(res.json())
